/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*-------------------------------------------------------------------------------------------------
                                        -- C N R S --
                      Laboratoire d'automatique et d'analyse des systemes
                        Groupe Robotique et Intelligenxce Artificielle
                                  7 Avenue du colonel Roche
                                    31 077 Toulouse Cedex

  Fichier              : tpFunctions.c
  Fonction             : Functions for Trajectory Planning 
  Date de creation     : Aout 2005      
  Date de modification : Aout 2005
  Nb de lignes         : 2451

  Auteur               : Ignacio HERRERA AGUILAR

  --------------------------------------------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include "matrixStruct.h"
#include "matrix.h"
#include "tpStruct.h"
#include "cFunctions.h"
#include "tpFunctions.h"
#include "tpConst.h"

#ifndef SQRT
#define SQRT(a)             (((a) < 0 ) ? (0) : (sqrt((a))))
#endif

/* Trajectory Planning Select Valable Time           
   tpSelectTime (dmat VR_Times, double TimeSelect)

   input:    VR_Times      Vector Row Time Solutions 

   output:   TimeSelect    Time Valable solution

   return:   TP_OK
             TP_ERROR
*/
TP_STATUS tpSelectTime (dmat VR_Times, double *TimeSelect)
{
  register int rowsize, colsize;
  register int i;
  TP_SELECT msgError;
  msgError = TP_OFF;
  
  rowsize = VR_Times.ub1 - VR_Times.lb1;
  colsize = VR_Times.ub2 - VR_Times.lb2;

  if (rowsize != 0 || colsize == 0) {
    fprintf (stderr, "tpSelectTime - TP_ERROR - Vector Dimensions \n");
    return TP_ERROR;
  }
  /* TimeSelect Initialitation */
  *TimeSelect = 1000.0;
  /* TimeSelect selection      */
  for (i=0;i<=colsize;i++)
    if (VR_Times.el[0][i]> 0.0 && VR_Times.el[0][i]<*TimeSelect) 
      *TimeSelect =  VR_Times.el[0][i];
  
  if (*TimeSelect == 1000.0 && !msgError) {
    fprintf(stderr,"tpSelectTime - There is not a valid solution - RETURN Time = 1000\n");
    matPrintf(VR_Times);
  }
    
  return TP_OK;
}

/* Adjust Time to Sampling Time Multiple
   tpSamplingAdjustTime(double Time, double aTime)
   
   input:   Time      Time Valor

   output:  aTime     Adjusted time

   return:  TP_OK
            TP_ERROR
*/
/* TP_STATUS tpSamplingAdjustTime(double Time, double *aTime) */
/* { */
/*   double auxTime; */
/*   double Ts; */
  
/*   auxTime = fmod(S_TD*Time,S_T); */

/*   Ts = (double) S_T / S_TD; */
/*   if (auxTime/S_TD > 0.5*Ts ) */
/*     *aTime = Time - auxTime/S_TD + Ts; */
/*   else */
/*     *aTime = Time - auxTime/S_TD; */
				  
/*   return TP_OK; */
/* } */

TP_STATUS tpSamplingAdjustTime(double Time, double *aTime)
{  
  *aTime = (int)(Time*NB_TICK_SEC) * SAMPLING_TIME;				  
  return TP_OK;
}

/* Verify Times values and goal distance 
   tpVerifyTimes(DISTANCE_TOLERANCE, double GD, TP_JERKS Jerks, TP_COND IC, int Dir, TP_TIMES T, TP_COND *FC, TP_TIMES *Acc, TP_TIMES *Vel, TP_TIMES *Pos, TP_SELECT msgError )
    
   Inputs:   GD         Goal Distance
             Jerks      Jerks
	     IC         Initial Conditions  A0, V0, X0
	     Dir        Direction
	     T          Times  = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
	     msgError   Display Error Messages TP_ON / TP_OFF

   Outputs:  FC         Final Conditions    AF, VF, XF
             Acc        Acceleration values
	     Vel        Velocity values
	     Pos        Position values
    
   Return:   TP_OK
             TP_ERROR      - Negative time
	                   - Fail Distance
*/
TP_STATUS tpVerifyTimes(double dist_error, double GD, TP_JERKS Jerks, TP_COND IC, int Dir, TP_TIMES T, TP_COND *FC, TP_TIMES *Acc, TP_TIMES *Vel, TP_TIMES *Pos, TP_SELECT msgError)
{
  double J1, J2, J3, J4;
  double Tjpa  = T.Tjpa;
  double Tjpa2 = Tjpa*Tjpa;
  double Tjpa3 = Tjpa*Tjpa2;
  double Taca  = T.Taca;
  double Taca2 = Taca*Taca;
  double Tjna  = T.Tjna;
  double Tjna2 = Tjna*Tjna;
  double Tjna3 = Tjna*Tjna2;
  double Tvc   = T.Tvc;
  double Tjnb  = T.Tjnb;
  double Tjnb2 = Tjnb*Tjnb;
  double Tjnb3 = Tjnb*Tjnb2;
  double Tacb  = T.Tacb;
  double Tacb2 = Tacb*Tacb;
  double Tjpb  = T.Tjpb;
  double Tjpb2 = Tjpb*Tjpb;
  double Tjpb3 = Tjpb*Tjpb2;
  double ErrorDistance = 0.0;
  
  /* Final Conditions Initialitation */
  FC->a = -1000;
  FC->v = -1000;
  FC->x = -1000;

  if (Tjpa<0) {
    if (!msgError)
      fprintf(stderr,"verify Times - TP_ERROR - Tjpa negative %f\n", Tjpa);
    return TP_ERROR;
  }
  if (Taca<0) {
    if (!msgError)
      fprintf(stderr,"verify Times - TP_ERROR - Taca negative %f\n",Taca);
    return TP_ERROR;
  }

  if (Tjna<0) {
    if (!msgError)
      fprintf(stderr,"verify Times - TP_ERROR - Tjna negative %f\n",Tjna);
    return TP_ERROR;
  }

  if (Tvc<0) {
    if (!msgError)
      fprintf(stderr,"verify Times - TP_ERROR - Tvc negative %f\n",Tvc);
    return TP_ERROR;
  }
  
  if (Tjnb<0) {
    if (!msgError)
      fprintf(stderr,"verify Times - TP_ERROR - Tjnb negative %f\n",Tjnb);
    return TP_ERROR;
  }

  if (Tacb<0) {
    if (!msgError)
      fprintf(stderr,"verify Times - TP_ERROR - Tacb negative %f\n",Tacb);
    return TP_ERROR;
  }

  if (Tjpb<0) {
    if (!msgError)
      fprintf(stderr,"verify Times - TP_ERROR - Tjpb negative %f\n",Tjpb);
    return TP_ERROR;
  }

  /* Jerks Option */
  if (Jerks.sel ==1) {
    J1 = Dir*Jerks.J1;
    J2 = Dir*Jerks.J1;
    J3 = Dir*Jerks.J1;
    J4 = Dir*Jerks.J1;
  } 
  else {
    J1 = Dir*Jerks.J1;
    J2 = Dir*Jerks.J2;
    J3 = Dir*Jerks.J3;
    J4 = Dir*Jerks.J4;
  }
  
  /* Acceleration, velocity and position evolution */
  Acc->Tjpa = IC.a      +        J1*Tjpa;
  Vel->Tjpa = IC.v      +      IC.a*Tjpa +        J1*Tjpa2/2;
  Pos->Tjpa = IC.x      +      IC.v*Tjpa +      IC.a*Tjpa2/2 + J1*Tjpa3/6;

  Acc->Taca = Acc->Tjpa;
  Vel->Taca = Vel->Tjpa + Acc->Tjpa*Taca;
  Pos->Taca = Pos->Tjpa + Vel->Tjpa*Taca + Acc->Tjpa*Taca2/2;

  Acc->Tjna = Acc->Taca -        J2*Tjna;
  Vel->Tjna = Vel->Taca + Acc->Taca*Tjna -        J2*Tjna2/2;
  Pos->Tjna = Pos->Taca + Vel->Taca*Tjna + Acc->Taca*Tjna2/2 - J2*Tjna3/6;

  Acc->Tvc  = Acc->Tjna;
  Vel->Tvc  = Vel->Tjna;
  Pos->Tvc  = Pos->Tjna + Vel->Tjna*Tvc;

  Acc->Tjnb = Acc->Tvc  -        J3*Tjnb;
  Vel->Tjnb = Vel->Tvc  +  Acc->Tvc*Tjnb -        J3*Tjnb2/2;
  Pos->Tjnb = Pos->Tvc  +  Vel->Tvc*Tjnb +  Acc->Tvc*Tjnb2/2 - J3*Tjnb3/6;
  
  Acc->Tacb = Acc->Tjnb;
  Vel->Tacb = Vel->Tjnb + Acc->Tjnb*Tacb;
  Pos->Tacb = Pos->Tjnb + Vel->Tjnb*Tacb + Acc->Tjnb*Tacb2/2;

  Acc->Tjpb = Acc->Tacb +        J4*Tjpb;
  Vel->Tjpb = Vel->Tacb + Acc->Tacb*Tjpb +        J4*Tjpb2/2;
  Pos->Tjpb = Pos->Tacb + Vel->Tacb*Tjpb + Acc->Tacb*Tjpb2/2 + J4*Tjpb3/6;

  /* Final Conditions */
  FC->a = Acc->Tjpb;
  FC->v = Vel->Tjpb;
  FC->x = Pos->Tjpb;

  /* Error Distance */
  ErrorDistance = Dir*GD + IC.x - Pos->Tjpb;
   
  if (fabs(ErrorDistance)> dist_error ) {
    if (!msgError) {
      fprintf(stderr,"Verify Times - TP_ERROR - Error Distance \n");
      fprintf(stderr,"ED = %f \n",ErrorDistance);
      printf("GD = %f \n",GD);
      printf("IC->a = %f\n",IC.a);
      printf("IC->v = %f\n",IC.v);
      printf("IC->x = %f\n",IC.x);
      printf("FC->a = %f\n",FC->a);
      printf("FC->v = %f\n",FC->v);
      printf("FC->x = %f\n",FC->x);
      printf("Dir = %d\n",Dir);
      printf("Jerk = %f\n",Jerks.J1);
      printf("Pos->Tjpa= %f\n",Pos->Tjpa);
      printf("Pos->Taca= %f\n",Pos->Taca);
      printf("Pos->Tjna= %f\n",Pos->Tjna);
      printf("Pos->Tvc= %f\n",Pos->Tvc);
      printf("Pos->Tjnb= %f\n",Pos->Tjnb);
      printf("Pos->Tacb= %f\n",Pos->Tacb);
      printf("Pos->Tjpb= %f\n",Pos->Tjpb);
      printf("Tjpa= %f\n",Tjpa);
      printf("Taca= %f\n",Taca);
      printf("Tjna= %f\n",Tjna);
      printf("Tvc= %f\n", Tvc);
      printf("Tjnb= %f\n",Tjnb);
      printf("Tacb= %f\n",Tacb);
      printf("Tjpb= %f\n",Tjpb);    
    }
    return TP_ERROR;
  }  
  return TP_OK;
}

/* Jerk Profile 
   tpJerkProfile(double Dist, double Jmax, double Amax, double Vmax, int Dir, double *Tj, double *Tac, double *Tvc, double *TimeL);
    
   Inputs:   Dist       Distance
             Jmax       Maximum Jerk
	     Amax       Maximum Acceleration
	     Vmax       Maximum Velocity
	     Dir        Direction
	
   Outputs:  Tj         Jerk Time
             Tac        Acceleration Constant Time
             Tvc        Velocity Constant Time
    
   Return:   TP_OK
             TP_ERROR      - Distance Negative
*/
TP_STATUS tpJerkProfile(double Dist, double Jmax, double Amax, double Vmax, double *Tj, double *Tac, double *Tvc, double *TimeL)
{
  double Jmax2 = Jmax*Jmax;
  double Amax2 = Amax*Amax;
  double Amax3 = Amax*Amax2;
  double Vmax2 = Vmax*Vmax;
  double Ta;
  double Thr1, Thr2;
  double Xdif, Xjp, Xjn;
 
  /* Temps Initialitation */
  *Tj  = 0.0;
  *Tac = 0.0;
  *Tvc = 0.0;
  *TimeL = 0.0;
  if (Dist == 0) return TP_OK;
  if (Dist < 0) {
    fprintf(stderr," tpJerkProfile - TP_ERROR - Distance negative \n");
    return TP_ERROR;
  }

  /* Acceleration Temps Calcul */
  *Tj  = Amax/Jmax;
  *Tac = Vmax/Amax - Amax/Jmax;
  Ta   =*Tac +  2*(*Tj);
  
  /* Calcul of Threshold 1 */
  Thr1 = Amax*Vmax/Jmax + Vmax2/Amax;

  /*  There are three main cases
      Case 1 : Distance = Threshold
      Case 2 : Distance > Threshold
      Case 3 : Distance < Threshold
      Calcul of a new Threshold (Threshold2)
      Case 3.1 : Distance = Threshold2
      Case 3.2 : Distance > Threshold2
      Case 3.3 : Distance < Threshold2  */

  if (Dist != Thr1) {
    if (Dist > Thr1) {
      /* Case 2 */
      Xdif = Dist - Thr1;
      *Tvc = Xdif/Vmax;
    }
    else {
      /* Case 3 */
      Xjp  = Amax3/Jmax2/6;    /* Distance with Positive Jerk */
      Xjn  = 5*Xjp;            /* Distance with Negative Jerk */ 
      /* Threshold 2 Calcul */
      Thr2 = Xjp + Xjn;
      if (Dist == Thr2) {
	/* Case 3.1 */
	*Tac = 0.0;
      }
      else {
	if (Dist/2 > Thr2) {
	  /* Case 3.2 */
	  Xdif = Dist/2 - Thr2;
	  *Tac = SQRT(0.25*Amax2/Jmax2 + Dist/Amax)-1.5*Amax/Jmax;
	}
	else {
	  *Tac = 0.0;
	  *Tj = cbrt(0.5*Dist/Jmax);
	}
      }
    }
  }
  *TimeL = 4*(*Tj) + 2*(*Tac) + *Tvc;
  return TP_OK;
}
/* Get Monotonic Times and Number of Elements
   tpGetMonotonicTimes(TP_TIMES T, TP_TIMES *TM, int *NOE)
   Inputs:   T          Times  = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
		   		   
   Outputs:  NOE        Number Of Elements
             TM         Times Monotonic = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb

   Return:   TP_OK
             TP_ERROR      - Sampling Adjust Time
	                   - Memory Failled
			   - Fail Distance
*/
TP_STATUS tpGetMonotonicTimes(TP_TIMES T,TP_TIMES *TM, int *NOE)
{
  double Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb;
  int auxNOE =0;
  
  *NOE = 0;

  /* Adjusting for Sampling Time */
  if (tpSamplingAdjustTime(T.Tjpa, &Tjpa)!= TP_OK) {
    fprintf(stderr, "tp Get Monotonic Times S D - TP_ERROR - tp Sampling Adjust Time Tjpa \n");
    return TP_ERROR;
  }
  if (tpSamplingAdjustTime(T.Taca, &Taca)!= TP_OK) {
    fprintf(stderr, "tp Get Monotonic Times S D - TP_ERROR - tp Sampling Adjust Time Taca \n");
    return TP_ERROR;
  }
  if (tpSamplingAdjustTime(T.Tjna, &Tjna)!= TP_OK) {
    fprintf(stderr, "tp Get Monotonic Times S D - TP_ERROR - tp Sampling Adjust Time Tjna \n");
    return TP_ERROR;
  }
  if (tpSamplingAdjustTime(T.Tvc, &Tvc)!= TP_OK) {
    fprintf(stderr, "tp Get Monotonic Times S D - TP_ERROR - tp Sampling Adjust Time Tvc \n");
    return TP_ERROR;
  }
  if (tpSamplingAdjustTime(T.Tjnb, &Tjnb)!= TP_OK) {
    fprintf(stderr, "tp Get Monotonic Times S D - TP_ERROR - tp Sampling Adjust Time Tjnb \n");
    return TP_ERROR;
  }
  if (tpSamplingAdjustTime(T.Tacb, &Tacb)!= TP_OK) {
    fprintf(stderr, "tp Get Monotonic Times S D - TP_ERROR - tp Sampling Adjust Time Tacb \n");
    return TP_ERROR;
  }
  if (tpSamplingAdjustTime(T.Tjpb, &Tjpb)!= TP_OK) {
    fprintf(stderr, "tp Get Monotonic Times S D - TP_ERROR - tp Sampling Adjust Time Tjpb \n");
    return TP_ERROR;
  }
  /* TJPA INTERVAL */
  if (Tjpa) {
    auxNOE = (Tjpa*S_TD)/S_T;
    if (ceil(auxNOE) - auxNOE > 0.0001)
      *NOE += ceil(auxNOE) - 1;
    else
      *NOE += ceil(auxNOE);
  }
  /* TACA INTERVAL */
  if (Taca) {
    auxNOE = (Taca*S_TD)/S_T;
    if (ceil(auxNOE) - auxNOE > 0.0001)
      *NOE += ceil(auxNOE) - 1;
    else
      *NOE += ceil(auxNOE);
  }
  /* TJNA INTERVAL */
  if (Tjna) {
    auxNOE = (Tjna*S_TD)/S_T;
    if (ceil(auxNOE) - auxNOE > 0.0001)
      *NOE += ceil(auxNOE) - 1;
    else
      *NOE += ceil(auxNOE);
  }
  /* TVC INTERVAL */
  if (Tvc) {
    auxNOE = (Tvc*S_TD)/S_T;
    if (ceil(auxNOE) - auxNOE > 0.0001)
      *NOE += ceil(auxNOE) - 1;
    else
      *NOE += ceil(auxNOE);
  }
  /* TJNB INTERVAL */
  if (Tjnb) {
    auxNOE = (Tjnb*S_TD)/S_T;
    if (ceil(auxNOE) - auxNOE > 0.0001)
      *NOE += ceil(auxNOE) - 1;
    else
      *NOE += ceil(auxNOE);
  }
  /* TACB INTERVAL */
  if (Tacb) {
    auxNOE = (Tacb*S_TD)/S_T;
    if (ceil(auxNOE) - auxNOE > 0.0001)
      *NOE += ceil(auxNOE) - 1;
    else
      *NOE += ceil(auxNOE);
  }
  /* TJPB INTERVAL */
  if (Tjpb) {
    auxNOE = (Tjpb*S_TD)/S_T;
    if (ceil(auxNOE) - auxNOE > 0.0001)
      *NOE += ceil(auxNOE);
    else
      *NOE += ceil(auxNOE)+1;
  }

  /* *NOE += 2; */
  
  TM->Tjpa = Tjpa;
  TM->Taca = Taca;
  TM->Tjna = Tjna;
  TM->Tvc  = Tvc;
  TM->Tjnb = Tjnb;
  TM->Tacb = Tacb;
  TM->Tjpb = Tjpb;

  return TP_OK;
}

/* Build the Acceleration, Velocity and Position Vectors in the Same Direction
   tpBuildVectorsSD(int Dir,TP_LIMITS Limits TP_JERKS Jerks, TP_COND IC, TP_TIMES T, TP_TIMES Acc, TP_TIMES Vel, TP_TIMES Pos, dmat Acc_V, dmat Vel_V, dmat Pos_V)
   Inputs:   Dir        Direction
             Limits     Limits: maxJerk, maxAcc, maxVel
	     Jerks      Jerks
	     IC         Initial Conditions  A0, V0, X0
	     TM         Adjusted Times: Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
	     Acc        Acceleration values for times
	     Vel        Velocity values for times
	     Pos        Position values for times
		   
   Outputs:  Acc_V      Acceleration  Vector
             Vel_V      Velocity Vector
	     Pos_V      Position Vector
    
   Return:   TP_OK
             TP_ERROR      - Sampling Adjust Time
	                   - Memory Failled
			   - Fail Distance
*/
TP_STATUS tpBuildVectorsSD(int Dir, TP_LIMITS Limits, TP_JERKS Jerks, TP_COND IC, TP_TIMES TM, TP_TIMES Acc, TP_TIMES Vel, TP_TIMES Pos, TP_VROW *Acc_V, TP_VROW *Vel_V, TP_VROW *Pos_V, int kBase)
{
  double J1, J2, J3, J4;
  double Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb;
  double Ts2, Ts3;
  int k = kBase;
  int i=0,j=0;
  int iMax;
  double k1, k2, k3;
  int NOE;                           /* Number of Elements */ 
  double auxIMax;

  double totalImax=0;
  /* Constants */
  Ts2 = (double)(S_T*S_T)/(S_TD*S_TD);
  Ts3 = (double)(S_T*Ts2)/S_TD;
  
  /* Jerks Option */
  if (Jerks.sel ==1) {
    J1 = Dir*Jerks.J1;
    J2 = Dir*Jerks.J1;
    J3 = Dir*Jerks.J1;
    J4 = Dir*Jerks.J1;
  } 
  else {
    J1 = Dir*Jerks.J1;
    J2 = Dir*Jerks.J2;
    J3 = Dir*Jerks.J3;
    J4 = Dir*Jerks.J4;
  }
  /* Times assignation */
  Tjpa = TM.Tjpa;
  Taca = TM.Taca;
  Tjna = TM.Tjna;
  Tvc  = TM.Tvc;
  Tjnb = TM.Tjnb;
  Tacb = TM.Tacb;
  Tjpb = TM.Tjpb;

  NOE = Pos_V->cols;

  /* Build Vectors */
  /* TJPA INTERVAL */
  if (Tjpa) {
    auxIMax = (Tjpa*S_TD)/S_T;
    if (ceil(auxIMax) - auxIMax > 0.0001)
      iMax = ceil(auxIMax) - 1;
    else {
      iMax = ceil(auxIMax);
    }
    totalImax += iMax ;
    for (i=0; i<iMax; i++) {
	 k1 = (double)(S_T*i)/S_TD;
	 k2 = i*i*Ts2/2;
	 k3 = i*i*i*Ts3/6;
	 Acc_V->el[0][k] = IC.a + J1*k1;
	 Vel_V->el[0][k] = IC.v + IC.a*k1 +   J1*k2;
	 Pos_V->el[0][k] = IC.x + IC.v*k1 + IC.a*k2 + J1*k3;
	 k++;
    }
  }
  /* TACA INTERVAL */
  if (Taca) {
    auxIMax = (Taca*S_TD)/S_T;
    if (ceil(auxIMax) - auxIMax > 0.0001)
      iMax = ceil(auxIMax) - 1;
    else {
      iMax = ceil(auxIMax);
    }
    totalImax += iMax ;
    for (i=0; i<iMax; i++) {
	 k1 = (double)(S_T*i)/S_TD;
	 k2 = i*i*Ts2/2;
	 Acc_V->el[0][k] = Acc.Tjpa;
	 Vel_V->el[0][k] = Vel.Tjpa + Acc.Tjpa*k1;
	 Pos_V->el[0][k] = Pos.Tjpa + Vel.Tjpa*k1 + Acc.Tjpa*k2;
	 k++;
    }
  }
  /* TJNA INTERVAL */
  if (Tjna) {
    auxIMax = (Tjna*S_TD)/S_T;
    if (ceil(auxIMax) - auxIMax > 0.0001)
      iMax = ceil(auxIMax) - 1;
    else {
      iMax = ceil(auxIMax);
    }
    totalImax += iMax ;
    for (i=0; i<iMax; i++) {
	 k1 = (double)(S_T*i)/S_TD;
	 k2 = i*i*Ts2/2;
	 k3 = i*i*i*Ts3/6;
	 Acc_V->el[0][k] = Acc.Taca - J2*k1;
	 Vel_V->el[0][k] = Vel.Taca + Acc.Taca*k1 -       J2*k2;
	 Pos_V->el[0][k] = Pos.Taca + Vel.Taca*k1 + Acc.Taca*k2 - J2*k3;
	 k++;
    }
  }
  /* TVC INTERVAL */
  if (Tvc) {
    auxIMax = (Tvc*S_TD)/S_T;
    if (ceil(auxIMax) - auxIMax > 0.0001)
      iMax = ceil(auxIMax) - 1;
    else {
      iMax = ceil(auxIMax);
    }
    totalImax += iMax ;
    for (i=0; i<iMax; i++) {
	 k1 = (double)(S_T*i)/S_TD;
	 Acc_V->el[0][k] = Acc.Tjna;
	 Vel_V->el[0][k] = Vel.Tjna;
	 Pos_V->el[0][k] = Pos.Tjna + Vel.Tjna*k1;
	 k++;
    }
  }
  /* TJNB INTERVAL */
  if (Tjnb) {
    auxIMax = (Tjnb*S_TD)/S_T;
    if (ceil(auxIMax) - auxIMax > 0.0001)
      iMax = ceil(auxIMax) - 1;
    else {
      iMax = ceil(auxIMax);
    }
    totalImax += iMax ;
    for (i=0; i<iMax; i++) {
	 k1 = (double)(S_T*i)/S_TD;
	 k2 = i*i*Ts2/2;
	 k3 = i*i*i*Ts3/6;
	 Acc_V->el[0][k] = Acc.Tvc -      J3*k1;
	 Vel_V->el[0][k] = Vel.Tvc + Acc.Tvc*k1 -      J3*k2;
	 Pos_V->el[0][k] = Pos.Tvc + Vel.Tvc*k1 + Acc.Tvc*k2 - J3*k3;
	 k++;
    }
  }
  /* TACB INTERVAL */
  if (Tacb) {
    auxIMax = (Tacb*S_TD)/S_T;
    if (ceil(auxIMax) - auxIMax > 0.0001)
      iMax = ceil(auxIMax) - 1;
    else {
      iMax = ceil(auxIMax);
    }
    totalImax += iMax ;
    for (i=0; i<iMax; i++) {
	 k1 = (double)(S_T*i)/S_TD;
	 k2 = i*i*Ts2/2;
	 Acc_V->el[0][k] = Acc.Tjnb;
	 Vel_V->el[0][k] = Vel.Tjnb + Acc.Tjnb*k1;
	 Pos_V->el[0][k] = Pos.Tjnb + Vel.Tjnb*k1 + Acc.Tjnb*k2;
	 k++;
    }
  }
  /* TJPB INTERVAL */
  if (Tjpb) {
    auxIMax = (Tjpb*S_TD)/S_T;
    if (ceil(auxIMax) - auxIMax > 0.0001)
      iMax = ceil(auxIMax) ;
    else {
      iMax = ceil(auxIMax)+1;
    }
    totalImax += iMax ;
    for (i=0; i<iMax; i++) {
	 k1 = (double)(S_T*i)/S_TD;
	 k2 = i*i*Ts2/2;
	 k3 = i*i*i*Ts3/6;
	 Acc_V->el[0][k] = Acc.Tacb + J4*k1;
	 Vel_V->el[0][k] = Vel.Tacb + Acc.Tacb*k1 +       J4*k2;
	 Pos_V->el[0][k] = Pos.Tacb + Vel.Tacb*k1 + Acc.Tacb*k2 + J4*k3;
	 k++;
	 
    }
  }



  if (k-1< NOE) {
    j=0;
    for (i=k;i<=NOE;i++) {
	 if (ABS(Acc_V->el[0][i-1])>(Limits.maxAcc/10))
	   Acc_V->el[0][i] = Acc_V->el[0][i-1]-Acc_V->el[0][i-1]*j/(NOE-k);
	 else 
	   Acc_V->el[0][i] = 0.0;
	 if (ABS(Vel_V->el[0][i-1])>(Limits.maxVel/10))
	   Vel_V->el[0][i] = Vel_V->el[0][i-1]- Vel_V->el[0][i-1]*j/(NOE-k);
	 else
	   Vel_V->el[0][i] = 0.0;
	 Pos_V->el[0][i] = Pos_V->el[0][k-1];
	 j++;
    }
  }
  return TP_OK;
}

/* Build the Reference Vectors after Jerk Correction considering fixed times 
   tpJerkProfileVectors(double GD,TP_COND IC, TP_TIMES T, TP_TIMES TM, int Dir, TP_LIMITS Limits, dmat Acc_R, dmat Vel_R, dmat Pos_R)
    
   Inputs:   GD         Goal Distance
             IC         Initial Conditions  A0, V0, X0
	     T          Times: Tjc Tac Tjc Tvc Tjc Tac Tjc
	     Tjc        Constant Jerk Time
	     Tac        Constant Acceleration Time
	     Tvc        Constant Velocity Time
	     TM         Monotonic Times: Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
	     Dir        Direction
	     Limits     Limits : maxJerk, maxAcc, maxVel;
	
   Outputs:  Acc_R      Acceleration Reference vector
             Vel_R      Velocity Reference vector
	     Pos_R      Position Reference vector
    
   Return:   TP_OK
             TP_ERROR      - Verify Times
	                   - Build Vectors
*/
TP_STATUS tpJerkProfileVectors(double GD,TP_COND IC, TP_TIMES T, TP_TIMES TM, int Dir, TP_LIMITS Limits, TP_VROW *VR_Acc, TP_VROW *VR_Vel, TP_VROW *VR_Pos, int kBase)
{
  double J1;
  double Tjc = T.Tjpa;
  double Tac = T.Taca;
  double Tvc = T.Tvc;
  double Tjc2 = Tjc*Tjc;
  double Tjc3 = Tjc*Tjc2;
  double Tac2 = Tac*Tac;
  TP_JERKS Jmax;
  TP_TIMES Acc, Vel, Pos;
  TP_COND FC;

  /*	CASE 1:   Tvc         Tac         Tjp
    CASE 2:   Tvc=0       Tac         Tjp
    CASE 3:   Tvc=0       Tac=0       Tjp
  */
  if (Tvc!=0) 
    J1 = GD/(Tjc*(2*Tjc2+3*Tjc*Tac+Tjc*Tvc+Tac*Tvc+Tac2));
  else
    if (Tac!=0)
      J1 = GD/(Tjc*(2*Tjc2+3*Tjc*Tac+Tac2));
    else
      J1 = 0.5*GD/Tjc3;
  
  Jmax.sel = 1;
  Jmax.J1  = J1;

  if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jmax, IC, Dir, T, &FC, &Acc, &Vel, &Pos, TP_ON)!=TP_OK) {
    fprintf(stderr,"tpJerkProfileVectors - TP_ERROR - Verify Times \n");
    return TP_ERROR;
  }

  if (tpBuildVectorsSD(Dir, Limits, Jmax, IC, TM, Acc, Vel, Pos, VR_Acc, VR_Vel, VR_Pos,kBase) != TP_OK) {
    fprintf(stderr,"tpJerkProfileVectors - TP_ERROR - Build Vectors \n");
    return TP_ERROR;
  }
  return TP_OK;
}

TP_STATUS tpGetNumberOfElement (TP_TIMES* TM, TP_TIMES* TNE)
{
  TNE->Tjpa = TM->Tjpa*(S_TD/S_T);
  TNE->Taca = TM->Taca*(S_TD/S_T);
  TNE->Tjna = TM->Tjna*(S_TD/S_T);
  TNE->Tvc  = TM->Tvc*(S_TD/S_T);
  TNE->Tjnb = TM->Tjnb*(S_TD/S_T);
  TNE->Tacb = TM->Tacb*(S_TD/S_T);
  TNE->Tjpb = TM->Tjpb*(S_TD/S_T);
  return TP_OK;
}

/* Like tpJerkProfileVectors but this function doesn't calcule the vectors Acc, Vel and Pos */
TP_STATUS tpJerkProfileAdjusted(double GD,TP_COND IC, TP_TIMES T, TP_TIMES TM, int Dir, TP_LIMITS Limits, TP_JERKS* Jmax, TP_TIMES* Acc, TP_TIMES* Vel, TP_TIMES* Pos)
{
  double J1;
  double Tjc = T.Tjpa;
  double Tac = T.Taca;
  double Tvc = T.Tvc;
  double Tjc2 = Tjc*Tjc;
  double Tjc3 = Tjc*Tjc2;
  double Tac2 = Tac*Tac;
//  TP_JERKS Jmax;
//  TP_TIMES Acc, Vel, Pos;
  TP_COND FC;

  /*	CASE 1:   Tvc         Tac         Tjp
    CASE 2:   Tvc=0       Tac         Tjp
    CASE 3:   Tvc=0       Tac=0       Tjp
  */
  if (Tvc!=0) 
    J1 = GD/(Tjc*(2*Tjc2+3*Tjc*Tac+Tjc*Tvc+Tac*Tvc+Tac2));
  else
    if (Tac!=0)
      J1 = GD/(Tjc*(2*Tjc2+3*Tjc*Tac+Tac2));
    else
      J1 = 0.5*GD/Tjc3;
  
  Jmax->sel = 1;
  Jmax->J1  = J1;

  if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, *Jmax, IC, Dir, T, &FC, Acc, Vel, Pos, TP_ON)!=TP_OK) {
    fprintf(stderr,"tpJerkProfileVectors - TP_ERROR - Verify Times \n");
    return TP_ERROR;
  }
  return TP_OK;
}
/* Calcul the distance needed for stoping
   tpStopPosition(TP_COND IC, TP_LIMITS Limits, double *StopPos)

   Inputs:   IC         Initial Conditions  A0, V0, X0
             Limits     Limits: maxJerk, maxAcc, maxVel

   Outputs:  StopPos    Stop Position
             T          Times Vector
    
   Return:   TP_OK
             TP_ERROR      - Verify Times
*/
TP_STATUS tpStopPosition(TP_COND IC, TP_LIMITS Limits, double *StopPos, TP_TIMES *T)
{
  double JMax = Limits.maxJerk;
  double JMax2= JMax*JMax;
  double AMax = Limits.maxAcc;
  double AMax2= AMax*AMax;
  double A0,A02,A03,A04;
  double V02;
  TP_JERKS Jerk;
  double D2;
  double V0;
  double Vc;                /* Velocity of control, limit for Tac = 0 */
  int    Dir;
  TP_COND FC;
  TP_TIMES Acc, Vel, Pos;

  Jerk.sel = 1;
  Jerk.J1  = JMax;

  /* Find direction */
  Dir = SIGN(IC.v);
  /* Initial Conditions correction */
  if (Dir<0) {
    A0 = Dir*IC.a;
    V0 = Dir*IC.v;
  }
  else {
    A0 = IC.a;
    V0 = IC.v;
  }

  A02 = A0*A0;
  A03 = A0*A02;
  A04 = A0*A03;
  
  V02 = V0*V0;

  /* Calcul of Velocity control */
  Vc = AMax2/JMax;
  T->Tjpa = 0.0;
  T->Taca = 0.0;
  T->Tjna = 0.0;
  T->Tvc  = 0.0;
  if ( ABS(V0) > Vc ) {
    T->Tjnb = (A0 + AMax)/JMax;
    if (T->Tjnb>-SAMPLING_TIME && T->Tjnb<0) {
      T->Tjnb= 0.0;
    }
    T->Tjpb = AMax/JMax;
    T->Tacb = (V0*JMax+0.5*A02-AMax2)/(JMax*AMax);
    D2     = V0*A0/JMax+V0*AMax/(2*JMax)+A03/(3*JMax2)+A02*AMax/(4*JMax2)+V02/(2*AMax)+V0*A02/(2*JMax*AMax)+A04/(8*JMax2*AMax);
  }
  else {
    T->Tacb = 0;
    T->Tjnb = (A0+SQRT(2*A02+4*V0*JMax)/2)/JMax;
    T->Tjpb = (-A0+JMax*T->Tjnb)/JMax;
    D2     = (12*V0*JMax*A0+(6*V0*JMax+3*A02)*SQRT(2*A02+4*V0*JMax)+4*A03)/(12*JMax2);
    if (T->Tjnb<0) {
      fprintf(stdout,"ARGHHH \n");
      T->Tjnb = 0.0;
      T->Tacb = -(2*V0*JMax-A02)/(2*JMax*A0);
      if (T->Tacb<0) {
	T->Tacb = 0.0;
	T->Tjpb =-A0/JMax;
      }
      T->Tjpb =-A0/JMax;
      D2 = -V02/(2*A0)-A03/(24*JMax2);
    }
  }
  
  if (tpVerifyTimes(DISTANCE_TOLERANCE, D2, Jerk, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_ON)!=TP_OK) {
    fprintf(stderr,"tpStopPosition - TP_ERROR - Verify Times \n");
    fprintf(stdout,"A0 %f, V0 %f, Tjnb %f, Tacb = %f, Tjpb = %f \n",A0, V0, T->Tjnb, T->Tacb, T->Tjpb);
    fprintf(stdout,"IC.a %f, IC.v %f \n", IC.a, IC.v);
    return TP_ERROR;
  }

  *StopPos = Dir*D2 + IC.x;

  /*  fprintf(stdout, "Acc F = %f,  Vel F = %f, Pos F = %f \n",FC.a,FC.v,*StopPos); */

  return TP_OK;
}


/* Return Times Structure for Jerk Profile Extention in the Same Direction deplacement
   tpJerkProfileESD(double GD, TP_LIMITS Limits, TP_COND IC, int Dir, TP_TIMES *T)
    
   Inputs:   GD         Goal Distance
             IC         Initial Conditions  A0, V0, X0
	     Dir        Direction
		  
   Outputs:  T          Times  = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
    
   Return:   TP_OK
             TP_ERROR      - Negative time
	     - Fail Distance
*/
TP_STATUS tpJerkProfileESD(double GD, TP_LIMITS Limits, TP_COND IC, int Dir, TP_TIMES *T)
{
  double Jmax = Limits.maxJerk;
  double Amax = Limits.maxAcc;
  double Vmax = Limits.maxVel;
  double Tjpa = 0.0;
  double Taca = 0.0;
  double Tjna = 0.0;
  double Tvc  = 0.0;
  double Tjnb = 0.0;
  double Tacb = 0.0;
  double Tjpb = 0.0;
  double A0   = 0.0;
  double V0   = 0.0;
  double X0   = 0.0;
  int register FlagCase = 0;
  double Vjp, Vac, Vjpa, Vc;
  double Xjpb, Aux, XRes, XRes2;
  double Tx;
  double Co4, Co3, Co2, Co1, Co0;
  double S0, S1, S2a, S2a3, S2,S3, S5;
  double S2p3, S2p6, S3p5, S321, T0, T1;
  double S2a1, S2a2, S2b1,S2b, S2c,S2d1,S2d2,S2d3,S2d;
  double S3a, S3a1,S3b,S3b1,P1,P2,P3,P3a,P4,P5,P6;
  double Xjpa1, Tjpa1, Ajpa1, Vjpa1, Tja, Tjb, Tjb2, Tjb3;
  double ITjpa1, ITjpa2, Ar, Vr;
  double Vjpa12, Vjpa13, Vjpa15;
  double FS1, FS1a, FS1p3, FS1p6, FS2, FS2p5, FS1FS2;
  double Jmax2 = Jmax * Jmax;
  double Jmax3 = Jmax * Jmax2;  
  double Jmax4 = Jmax * Jmax3;
  double Jmax5 = Jmax * Jmax4;
  double Jmax6 = Jmax * Jmax5;  
  double Jmax7 = Jmax * Jmax6;
  double Jmax8 = Jmax * Jmax7;
  double Jmax9 = Jmax * Jmax8;
  double Amax2 = Amax * Amax;
  double Amax3 = Amax * Amax2;
  double Amax4 = Amax * Amax3;
  double Vmax2 = Vmax * Vmax;
  double A02, A03, A04, A05, A06, A07, A08, A09, A010;
  double A011, A012, A013, A014, A015, A016;
  double V02, V03, V04, V05, V06, V07, V08;
  double GD2   = GD*GD;
  double GD3   = GD*GD2;
  double GD4   = GD*GD3;
  double K1, K2;
  double K3p6  = cbrt(9);
  double K3p3  = cbrt(3);
  int errno;
  dmat VR_Pol, VR_Sol;
  TP_JERKS Jerks;
  TP_COND FC;
  TP_TIMES Acc, Vel, Pos;
  
  VR_Pol = newdmat(0,0,0,4,&errno);
  if (errno) {
    fprintf(stderr,"tp Jerk Profile ESD - TP_ERROR - Definition VR_Pol \n");
    return TP_ERROR;
  }
  VR_Sol = newdmat(0,0,0,3,&errno);
  if (errno) {
    fprintf(stderr,"tp Jerk Profile ESD - TP_ERROR - Definition VR_Sol \n");
    return TP_ERROR;
  }

  Jerks.sel = 1;
  Jerks.J1  = Jmax;
 
  /* Positive Deplacement conditions */
  A0 = SIGN(IC.v)*IC.a;
  if (A0 > - ACCELERATION_ZERO && A0< ACCELERATION_ZERO)
    A0 = 0.0;
  V0 = SIGN(IC.v)*IC.v;
  A02   = A0*A0;
  A03   = A0*A02;
  A04   = A0*A03;
  A05   = A0*A04;
  A06   = A0*A05;
  A07   = A0*A06;
  A08   = A0*A07;
  A09   = A0*A08;
  A010  = A0*A09;
  A011  = A0*A010;
  A012  = A0*A011;
  A013  = A0*A012;
  A014  = A0*A013;
  A015  = A0*A014;
  A016  = A0*A015;
  V02   = V0*V0;
  V03   = V0*V02;
  V04   = V0*V03;
  V05   = V0*V04;
  V06   = V0*V05;
  V07   = V0*V06;
  V08   = V0*V07;
  
  /* Velocity thresholds */
  Vjp = Amax2/(2*Jmax); 
  Vac = Vmax - Vjp;
  
  if ( A0!=0.0 || V0!=0.0) { /* IF ( A0!=0.0 || V0!=0.0 ) ===================================================================(01 i) */
    if ( V0 == Vmax ) { /* IF ( V0 == Vmax ) ================================================================================(02 i) */
      FlagCase = 1;
      Tjnb = (A0+Amax)/Jmax;
      Tacb = (Vmax*Jmax + A02/2 - Amax2)/(Jmax*Amax);
      Tjpb = Amax/Jmax;
      Xjpb = (24*Amax*Vmax*Jmax*A0+12*Vmax*Jmax*Amax2+8*Amax*A03+6*A02*Amax2+12*Vmax2*Jmax2+12*Vmax*Jmax*A02+3*A04)/(24*Jmax2*Amax);
      Tvc  = (GD-Xjpb)/V0;
    } else { /* ELSE ( V0 == Vmax ) =========================================================================================(02 e) */
      if (fabs(A0- Amax)<0.00001) { /* IF ( A0 == Amax ) ====================================================================(03 i) */
	Tjpa = 0.0;
	Xjpb = -V02/(2*Amax) -Amax3/(24*Jmax2)+Amax*Vmax/Jmax+Vmax2/Amax;
	if (GD >= Xjpb) { /* IF  (GD >= Xjpb) ===============================================================================(04 i) */
	  FlagCase = 2;
	  Taca = (-2*V0*Jmax-Amax2+2*Vmax*Jmax)/(2*Jmax*Amax);
	  if (fabs(Taca)<0.00001)
	    Taca = 0.0;
	  Tjna = Amax/Jmax;
	  Tvc  = (GD-Xjpb)/Vmax;
	  Tjnb = Amax/Jmax;
	  Tacb = (-Amax2+Vmax*Jmax)/(Jmax*Amax);
	  Tjpb = Tjnb;
	} else {  /* ELSE  (GD >= Xjpb) =====================================================================================(04 e) */
	  FlagCase = 3;
	  Tjpa = 0;
	  Taca = (-Amax2-V0*Jmax+SQRT(42*Amax4+72*V02*Jmax2+144*GD*Amax*Jmax2)/12)/(Amax*Jmax);
	  Tjna = Amax/Jmax;
	  Tvc  = 0;
	  Tjnb = Tjna;
	  Tacb = (2*V0*Jmax+2*Amax*Taca*Jmax-Amax2)/(2*Jmax*Amax);
	  Tjpb = Tjnb;
	} /*  END  (GD >= Xjpb) =============================================================================================(04 n) */
      } else { /* ELSE ( A0 == Amax ) =======================================================================================(03 e) */
	Tjpa = (Amax - A0)/Jmax;
	Vjpa = V0 + A0*Tjpa + Jmax*Tjpa*Tjpa/2;
	if ( Vjpa <= Vac ) { /* IF ( Vjpa <= Vac ) ==========================================================================(05 i) */
	  Taca = (Vac-Vjpa)/Amax;
	  Tjna = Amax/Jmax;
	  Tjnb = Tjna;
	  Tacb = Vmax/Amax - Amax/Jmax;
	  Tjpb = Tjnb;
	  Xjpb = Amax*V0/(2*Jmax)-V0*A0/Jmax-Amax*A02/(4*Jmax2)+A03/(3*Jmax2)-V02/(2*Amax)+V0*A02/(2*Jmax*Amax)-A04/(8*Jmax2*Amax)+Amax*Vmax/Jmax+Vmax2/Amax;
	} else {  /* ELSE ( Vjpa <= Vac ) ===================================================================================(05 e) */
	  Aux  = SQRT(2*A02-4*V0*Jmax+4*Vmax*Jmax);
	  if (Aux<0.00001 && Aux>-0.00001)
	    Aux=0;
	  Tjpa = (-A0+Aux/2)/Jmax;
	  if (Tjpa<0.00001 && Tjpa>-0.00001)
	    Tjpa=0;
	  Taca = 0;
	  Tjna = (A0+Jmax*Tjpa)/Jmax;
	  if (Tjna<0.00001 && Tjna>-0.00001)
	    Tjna=0;
	  Tjnb = Amax/Jmax;
	  Tacb = Vmax/Amax - Amax/Jmax;
	  Tjpb = Tjnb;
	  Xjpb = (Aux/(2*Jmax)-A0/Jmax)*V0+Aux*Vmax/(2*Jmax)+A03/(3*Jmax2)-A02*Aux/(4*Jmax2)+Vmax*Amax/(2*Jmax)+Vmax2/(2*Amax);
	} /* END ( Vjpa <= Vac ) ============================================================================================(05 n) */
	if ( GD >= Xjpb ) { /* IF ( GD >= Xjpb ) ============================================================================(06 i) */
	  FlagCase = 4;
	  Tvc = (GD-Xjpb)/Vmax;
	} else { /* ELSE ( GD >= Xjpb ) =====================================================================================(06 e) */
	  Tjpa = (Amax-A0)/Jmax;
	  Vjpa = V0 + A0*Tjpa + Jmax*Tjpa*Tjpa/2;
	  if ( Vjpa <= Vac ) { /* IF ( Vjpa <= Vac ) ========================================================================(07 i) */
	    Tjna = Amax/Jmax;
	    Tjnb = Tjna;
	    Taca = 0;
	    Tacb = (2*V0*Jmax-A02+2*Amax*Taca*Jmax)/(2*Jmax*Amax);
	    Tjpb = Tjna;
	    Xjpb = (84*V0*Jmax*Amax2-24*Amax*V0*Jmax*A0-42*A02*Amax2+8*Amax*A03+48*Amax4+12*V02*Jmax2-12*V0*Jmax*A02+3*A04)/(24*Jmax2*Amax);
	    if ( GD >= Xjpb ) { /* IF ( GD >= Xjpb ) ========================================================================(08 i) */
	      FlagCase = 5;
	      Taca = (-V0*Jmax-3*Amax2/2+A02/2+SQRT(72*V02*Jmax2-72*V0*Jmax*Amax2-72*V0*Jmax*A02+36*Amax4+36*A02*Amax2+18*A04+144*Amax*V0*Jmax*A0-48*Amax*A03+144*GD*Amax*Jmax2)/12)/(Jmax*Amax);
	      Tacb = (2*V0*Jmax-A02+2*Amax*Taca*Jmax)/(2*Jmax*Amax);
	    } else /* ELSE ( GD >= Xjpb ) ===================================================================================(08 e) */
	      FlagCase = 6;
	    /* END ( GD >= Xjpb ) ===========================================================================================(08 n) */
	  } else /* ELSE ( Vjpa <= Vac ) ====================================================================================(07 e) */
	    FlagCase = 6;
	  /* END ( Vjpa <= Vac ) ============================================================================================(07 n) */
	  if (FlagCase == 6) { /* IF ( FlagCase == 6 ) ======================================================================(09 i) */
	    Co4 = Jmax2/(2*Amax);
	    Co3 = 2*Jmax*A0/Amax+Jmax;
	    Co2 = Jmax*V0/Amax+Amax/2+ 5*A02/(2*Amax)+3*A0;
	    Co1 = A0*Amax/Jmax+A03/(Jmax*Amax)+2*A02/Jmax+2*A0*V0/Amax+2*V0;
	    Co0 = Amax*V0/(2*Jmax)+A02*V0/(2*Jmax*Amax)+A0*V0/Jmax+Amax*A02/(4*Jmax2)+A04/(8*Jmax2*Amax)+A03/(3*Jmax2)+V02/(2*Amax);
	    VR_Pol.el[0][0] = Co4;
	    VR_Pol.el[0][1] = Co3;
	    VR_Pol.el[0][2] = Co2;
	    VR_Pol.el[0][3] = Co1;
	    VR_Pol.el[0][4] = Co0 - GD;
	    if (quarticRSolution(VR_Pol, VR_Sol, TP_OFF) != TP_OK) {
	      fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - quarticSolution 09 i\n");
	      return TP_ERROR;
	    }
	    if(tpSelectTime(VR_Sol,&Tjpa)!=TP_OK) {
	      fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - select Time     09 i\n");
	      return TP_ERROR;
	    }
	    if (Tjpa<0.00001 && Tjpa>-0.00001)
	      Tjpa=0;
	    Tjna = (A0+Jmax*Tjpa)/Jmax;
	    if (Tjna<0.00001 && Tjna>-0.00001)
	      Tjna=0;
	    Tjnb = Amax/Jmax;
	    Tacb = (2*V0*Jmax+4*A0*Tjpa*Jmax+2*Jmax2*Tjpa*Tjpa+A02-2*Amax2)/(2*Jmax*Amax);
	    Tjpb = Tjnb;
	    T->Tjpa = Tjpa;
	    T->Taca = 0.0;
	    T->Tjna = Tjna;
	    T->Tvc  = 0.0;
            T->Tjnb = Tjnb;
            T->Tacb = Tacb;
	    T->Tjpb = Tjpb;
	    if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_OFF ) != TP_OK) { /* IF (Verify Times) =============(10 i) */
	      FlagCase = 7;
	      S0 = (2*V0*Jmax-A02);
	      if (S0 == 0)
		S0 = 0.000001;
	      S1 = S0*S0*S0;
	      S2a= 72*Jmax4*GD2+144*Jmax3*A0*V0*GD-48*Jmax2*A03*GD+72*V03*Jmax3-36*V02*Jmax2*A02+6*V0*Jmax*A04-A06 ;
	      S2a3 = S2a*S2a*S2a;
	      S2 = (31104*GD4*Jmax8+51840*V03*GD2*Jmax7+124416*GD3*Jmax7*A0*V0+21312*V06*Jmax6+103680*A0*GD*Jmax6*V04+108864*V02*GD2*Jmax6*A02-41472*GD3*Jmax6*A03-12096*V05*Jmax5*A02-65664*A03*GD*Jmax5*V03-85536*V0*GD2*Jmax5*A04-1296*V04*Jmax4*A04+5184*A05*GD*Jmax4*V02+14256*Jmax4*A06*GD2+1728*V03*Jmax3*A06+2592*A07*GD*Jmax3*V0-324*V02*Jmax2*A08-288*A09*GD*Jmax2+36*A010*V0*Jmax-3*A012+2*SQRT(3)*SQRT((216*Jmax4*GD2-144*Jmax2*A03*GD-A06+432*Jmax3*A0*V0*GD+6*V0*Jmax*A04-84*V02*Jmax2*A02+200*V03*Jmax3)*S2a3))*S1*S1;
	      S2p3 = cbrt(S2);
	      S2p6 = cbrt(S2*S2);
	      S3 = -(28*S2p3*A08+16896*Jmax8*K3p6*V08+192*Jmax4*V04*S2p3+2*K3p6*A016+36864*Jmax8*K3p6*A0*GD*V06-46080*Jmax8*K3p6*GD2*A02*V04-104448*Jmax7*K3p6*A03*GD*V05+46080*Jmax7*K3p6*GD2*A04*V03+122880*Jmax6*K3p6*A05*GD*V04-23040*Jmax6*K3p6*GD2*A06*V02-76800*Jmax5*K3p6*A07*GD*V03+5760*Jmax5*V0*K3p6*GD2*A08+26880*Jmax4*V02*K3p6*A09*GD-32*Jmax*V0*K3p6*A014+18432*K3p6*GD2*V05*Jmax9-49152*Jmax7*K3p6*A02*V07+59904*Jmax6*K3p6*A04*V06-39424*Jmax5*K3p6*A06*V05-4992*Jmax3*V0*K3p6*A011*GD+15040*Jmax4*V04*K3p6*A08-576*Jmax4*K3p6*GD2*A010-3328*Jmax3*V03*K3p6*A010+416*Jmax2*V02*K3p6*A012+384*Jmax2*K3p6*A013*GD-2*K3p3*S2p6-576*Jmax4*V02*S2p3*A0*GD-224*Jmax*V0*S2p3*A06-288*Jmax5*V0*S2p3*GD2+144*Jmax4*S2p3*GD2*A02-672*Jmax3*V03*S2p3*A02+624*Jmax2*V02*S2p3*A04-96*Jmax2*S2p3*A05*GD+480*Jmax3*V0*S2p3*A03*GD)/(S2p3*S1);
	      S3p5 = SQRT(S3);
	      T0 = (-144*GD*Jmax2+144*V0*Jmax*A0-96*A03);
	      T1 = -T0/(4*(72*V0*Jmax-36*A02));
	      S321  = S3p5*S2p3*S1;
	      VR_Sol.el[0][0] = (T1+S3p5/12+SQRT(2)*SQRT((-S3p5*K3p3*S2p6-28*S2p3*S3p5*A08+1728*Jmax6*S2p3*GD3+S3p5*K3p6*A016+8448*Jmax8*S3p5*K3p6*V08+432*Jmax3*V0*S2p3*GD*A04+5184*Jmax5*V0*S2p3*GD2*A0-2592*Jmax4*V02*S2p3*GD*A02-192*Jmax4*V04*S2p3*S3p5+5184*Jmax5*V03*S2p3*GD+5184*Jmax4*V04*S2p3*A0-1728*Jmax4*S2p3*GD2*A03-7776*Jmax3*V03*S2p3*A03+4752*Jmax2*V02*S2p3*A05-72*Jmax2*S2p3*GD*A06-1368*Jmax*V0*S2p3*A07+672*Jmax3*V03*S2p3*S3p5*A02-480*Jmax3*V0*S2p3*S3p5*A03*GD-624*Jmax2*V02*S2p3*S3p5*A04-144*Jmax4*S2p3*S3p5*GD2*A02+288*Jmax5*V0*S2p3*S3p5*GD2+576*Jmax4*V02*S2p3*S3p5*A0*GD+96*Jmax2*S2p3*S3p5*A05*GD+224*Jmax*V0*S2p3*S3p5*A06+152*S2p3*A09+29952*Jmax6*S3p5*K3p6*A04*V06+61440*Jmax6*S3p5*K3p6*A05*GD*V04-11520*Jmax6*S3p5*K3p6*GD2*A06*V02-19712*Jmax5*S3p5*K3p6*A06*V05-38400*Jmax5*V03*S3p5*K3p6*A07*GD+2880*Jmax5*V0*S3p5*K3p6*GD2*A08+9216*S3p5*K3p6*GD2*V05*Jmax9+18432*Jmax8*S3p5*K3p6*A0*GD*V06-24576*Jmax7*S3p5*K3p6*A02*V07-23040*Jmax8*S3p5*K3p6*GD2*A02*V04-52224*Jmax7*S3p5*K3p6*A03*GD*V05+23040*Jmax7*S3p5*K3p6*GD2*A04*V03-2496*Jmax3*V0*S3p5*K3p6*A011*GD+208*Jmax2*V02*S3p5*K3p6*A012+192*Jmax2*S3p5*K3p6*A013*GD-1664*Jmax3*V03*S3p5*K3p6*A010+7520*Jmax4*V04*S3p5*K3p6*A08+13440*Jmax4*V02*S3p5*K3p6*A09*GD-288*Jmax4*S3p5*K3p6*GD2*A010-16*Jmax*V0*S3p5*K3p6*A014)/S321)/12)/Jmax;
	      VR_Sol.el[0][1] = (T1+S3p5/12-SQRT(2)*SQRT((432*Jmax3*V0*S2p3*GD*A04+5184*Jmax5*V0*S2p3*GD2*A0-2592*Jmax4*V02*S2p3*GD*A02+672*Jmax3*V03*S2p3*S3p5*A02-480*Jmax3*V0*S2p3*S3p5*A03*GD-624*Jmax2*V02*S2p3*S3p5*A04+1728*Jmax6*S2p3*GD3-S3p5*K3p3*S2p6-28*S2p3*S3p5*A08-72*Jmax2*S2p3*GD*A06-1368*Jmax*V0*S2p3*A07-192*Jmax4*V04*S2p3*S3p5-144*Jmax4*S2p3*S3p5*GD2*A02+288*Jmax5*V0*S2p3*S3p5*GD2+576*Jmax4*V02*S2p3*S3p5*A0*GD+96*Jmax2*S2p3*S3p5*A05*GD+224*Jmax*V0*S2p3*S3p5*A06+5184*Jmax5*V03*S2p3*GD+5184*Jmax4*V04*S2p3*A0-1728*Jmax4*S2p3*GD2*A03-7776*Jmax3*V03*S2p3*A03+4752*Jmax2*V02*S2p3*A05+8448*Jmax8*S3p5*K3p6*V08+152*S2p3*A09+S3p5*K3p6*A016+29952*Jmax6*S3p5*K3p6*A04*V06+61440*Jmax6*S3p5*K3p6*A05*GD*V04-11520*Jmax6*S3p5*K3p6*GD2*A06*V02-19712*Jmax5*S3p5*K3p6*A06*V05-38400*Jmax5*V03*S3p5*K3p6*A07*GD+2880*Jmax5*V0*S3p5*K3p6*GD2*A08+9216*S3p5*K3p6*GD2*V05*Jmax9+18432*Jmax8*S3p5*K3p6*A0*GD*V06-24576*Jmax7*S3p5*K3p6*A02*V07-23040*Jmax8*S3p5*K3p6*GD2*A02*V04-52224*Jmax7*S3p5*K3p6*A03*GD*V05+23040*Jmax7*S3p5*K3p6*GD2*A04*V03-2496*Jmax3*V0*S3p5*K3p6*A011*GD+208*Jmax2*V02*S3p5*K3p6*A012+192*Jmax2*S3p5*K3p6*A013*GD-1664*Jmax3*V03*S3p5*K3p6*A010+7520*Jmax4*V04*S3p5*K3p6*A08+13440*Jmax4*V02*S3p5*K3p6*A09*GD-288*Jmax4*S3p5*K3p6*GD2*A010-16*Jmax*V0*S3p5*K3p6*A014)/S321)/12)/Jmax;
	      VR_Sol.el[0][2] = (T1-S3p5/12+SQRT(-(3456*Jmax6*S2p3*GD3-1344*Jmax3*V03*S2p3*S3p5*A02+960*Jmax3*V0*S2p3*S3p5*A03*GD+1248*Jmax2*V02*S2p3*S3p5*A04+288*Jmax4*S2p3*S3p5*GD2*A02-576*Jmax5*V0*S2p3*S3p5*GD2-1152*Jmax4*V02*S2p3*S3p5*A0*GD-192*Jmax2*S2p3*S3p5*A05*GD-448*Jmax*V0*S2p3*S3p5*A06+864*Jmax3*V0*S2p3*GD*A04+10368*Jmax5*V0*S2p3*GD2*A0-5184*Jmax4*V02*S2p3*GD*A02+2*S3p5*K3p3*S2p6+56*S2p3*S3p5*A08+10368*Jmax5*V03*S2p3*GD+10368*Jmax4*V04*S2p3*A0-3456*Jmax4*S2p3*GD2*A03-15552*Jmax3*V03*S2p3*A03+9504*Jmax2*V02*S2p3*A05-144*Jmax2*S2p3*GD*A06+384*Jmax4*V04*S2p3*S3p5-2736*Jmax*V0*S2p3*A07-2*S3p5*K3p6*A016-16896*Jmax8*S3p5*K3p6*V08+304*S2p3*A09-59904*Jmax6*S3p5*K3p6*A04*V06-122880*Jmax6*S3p5*K3p6*A05*GD*V04+23040*Jmax6*S3p5*K3p6*GD2*A06*V02+39424*Jmax5*S3p5*K3p6*A06*V05+76800*Jmax5*V03*S3p5*K3p6*A07*GD-5760*Jmax5*V0*S3p5*K3p6*GD2*A08-18432*S3p5*K3p6*GD2*V05*Jmax9-36864*Jmax8*S3p5*K3p6*A0*GD*V06+49152*Jmax7*S3p5*K3p6*A02*V07+46080*Jmax8*S3p5*K3p6*GD2*A02*V04+104448*Jmax7*S3p5*K3p6*A03*GD*V05-46080*Jmax7*S3p5*K3p6*GD2*A04*V03+4992*Jmax3*V0*S3p5*K3p6*A011*GD-416*Jmax2*V02*S3p5*K3p6*A012-384*Jmax2*S3p5*K3p6*A013*GD+3328*Jmax3*V03*S3p5*K3p6*A010-15040*Jmax4*V04*S3p5*K3p6*A08-26880*Jmax4*V02*S3p5*K3p6*A09*GD+576*Jmax4*S3p5*K3p6*GD2*A010+32*Jmax*V0*S3p5*K3p6*A014)/S321)/12)/Jmax;
	      VR_Sol.el[0][3] = (T1-S3p5/12-SQRT(-(-2*S3p5*K3p6*A016+3456*Jmax6*S2p3*GD3+304*S2p3*A09-16896*Jmax8*S3p5*K3p6*V08-59904*Jmax6*S3p5*K3p6*A04*V06-122880*Jmax6*S3p5*K3p6*A05*GD*V04+23040*Jmax6*S3p5*K3p6*GD2*A06*V02+39424*Jmax5*S3p5*K3p6*A06*V05+76800*Jmax5*V03*S3p5*K3p6*A07*GD-5760*Jmax5*V0*S3p5*K3p6*GD2*A08-18432*S3p5*K3p6*GD2*V05*Jmax9-36864*Jmax8*S3p5*K3p6*A0*GD*V06+49152*Jmax7*S3p5*K3p6*A02*V07+46080*Jmax8*S3p5*K3p6*GD2*A02*V04+104448*Jmax7*S3p5*K3p6*A03*GD*V05-46080*Jmax7*S3p5*K3p6*GD2*A04*V03-1344*Jmax3*V03*S2p3*S3p5*A02+864*Jmax3*V0*S2p3*GD*A04+10368*Jmax5*V0*S2p3*GD2*A0-5184*Jmax4*V02*S2p3*GD*A02+960*Jmax3*V0*S2p3*S3p5*A03*GD+1248*Jmax2*V02*S2p3*S3p5*A04+288*Jmax4*S2p3*S3p5*GD2*A02-576*Jmax5*V0*S2p3*S3p5*GD2-1152*Jmax4*V02*S2p3*S3p5*A0*GD+2*S3p5*K3p3*S2p6+56*S2p3*S3p5*A08-192*Jmax2*S2p3*S3p5*A05*GD-448*Jmax*V0*S2p3*S3p5*A06+384*Jmax4*V04*S2p3*S3p5+10368*Jmax5*V03*S2p3*GD+10368*Jmax4*V04*S2p3*A0-3456*Jmax4*S2p3*GD2*A03-15552*Jmax3*V03*S2p3*A03+9504*Jmax2*V02*S2p3*A05-144*Jmax2*S2p3*GD*A06-2736*Jmax*V0*S2p3*A07+4992*Jmax3*V0*S3p5*K3p6*A011*GD-416*Jmax2*V02*S3p5*K3p6*A012-384*Jmax2*S3p5*K3p6*A013*GD+3328*Jmax3*V03*S3p5*K3p6*A010-15040*Jmax4*V04*S3p5*K3p6*A08-26880*Jmax4*V02*S3p5*K3p6*A09*GD+576*Jmax4*S3p5*K3p6*GD2*A010+32*Jmax*V0*S3p5*K3p6*A014)/S321)/12)/Jmax;
	      if(tpSelectTime(VR_Sol,&Tjpa)!=TP_OK) {
		fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - select Time     10 i\n");
		return TP_ERROR;
	      }
/* 	      matPrintf(VR_Sol); */
/* 	      fprintf(stderr,"AQUI 0 %f \n",Tjpa); */
	      Tacb = 0;
	      Tjna = (A0+Jmax*Tjpa)/Jmax;
	      Tjnb = SQRT(4*V0*Jmax+8*A0*Tjpa*Jmax+4*Jmax2*Tjpa*Tjpa+2*A02)/(2*Jmax);
	      Tjpb = Tjnb;
	      T->Tjpa = Tjpa;
	      T->Taca = 0.0;
	      T->Tjna = Tjna;
	      T->Tvc  = 0.0;
	      T->Tjnb = Tjnb;
	      T->Tacb = Tacb;
	      T->Tjpb = Tjpb;
	      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_OFF) != TP_OK) { /* IF (Verify Times) ===========(11 i) */
		FlagCase=8;
		Co4 = Jmax*(2*V0*Jmax-A02)/2;
		Co3 = 2*Jmax*V0*A0-2*GD*Jmax2-4*A03/3;
		Co2 = (Jmax2*V02-6*GD*Jmax2*A0-3*A04/4-A02*V0*Jmax)/Jmax;
		Co1 = -(4*GD*Jmax3*V0+4*GD*A02*Jmax2+2*A03*V0*Jmax/3+A05/6+2*A0*V02*Jmax2)/Jmax2;
		Co0 = GD2*Jmax-2*GD*A03/(3*Jmax)-2*GD*A0*V0-A02*V02/(2*Jmax)-A04*V0/(12*Jmax2)-A06/(72*Jmax3)-V03;
		VR_Pol.el[0][0] = Co4;
		VR_Pol.el[0][1] = Co3;
		VR_Pol.el[0][2] = Co2;
		VR_Pol.el[0][3] = Co1;
		VR_Pol.el[0][4] = Co0;
		if (quarticRSolution(VR_Pol, VR_Sol, TP_OFF) != TP_OK) {
		  fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - quarticSolution 11 i\n");
		  return TP_ERROR;
		}
		if(tpSelectTime(VR_Sol,&Tjpa)!=TP_OK) {
		  fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - select Time     11 i\n");
		  return TP_ERROR;
		}
/* 		matPrintf(VR_Sol); */
/* 		fprintf(stderr,"AQUI 1 %f \n",Tjpa); */

		Tvc  = 0.0;
		Taca = 0.0;
		Tacb = 0.0;
		Vc   = V0+2*A0*Tjpa+Jmax*Tjpa*Tjpa+A02/(2*Jmax);
		Tjna = (A0+Jmax*Tjpa)/Jmax;
		if (Tjna<0)
		  Tjna = 0.0;
		Tjnb = SQRT(Vc/Jmax);
		Tjpb=Tjnb;
	      } /* END (Verify Times) =======================================================================================(11 n) */ 
	    } /* END (Verify Times) =========================================================================================(10 n) */
	  } /* END ( FlagCase == 6 ) ========================================================================================(09 i) */
	} /* END ( GD >= Xjpb ) =============================================================================================(06 n) */
      } /* END ( A0 == Amax ) ===============================================================================================(03 n) */
    } /* END ( V0 == Vmax ) =================================================================================================(02 n) */
  } /* END  ( A0!=0.0 || V0!=0.0 ) ==========================================================================================(01 n) */
    T->Tjpa = Tjpa;
    T->Taca = Taca;
    T->Tjna = Tjna;
    T->Tvc  = Tvc;
    T->Tjnb = Tjnb;
    T->Tacb = Tacb;
    T->Tjpb = Tjpb;
    if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_OFF ) != TP_OK ) { /* IF (Verify Times) =====================(12 i) */
      FlagCase = 9;
      Tvc = 0.0;
      Tjpa1 = -A0/Jmax;
      Ajpa1 = A0 + Jmax*Tjpa1;
      Vjpa1 = V0 + A0*Tjpa1 + Jmax*Tjpa1*Tjpa1/2;
      if (Vjpa1<0.000001) 
	Vjpa1=0;
      Xjpa1 = X0 + V0*Tjpa1 + A0*Tjpa1*Tjpa1/2 + Jmax*Tjpa1*Tjpa1*Tjpa1/6;
      Taca = 0;
      Tjna = 0;
      if (Vjpa1<=Vjp*2) {
        Tacb = 0;
        Tjb  = SQRT(Jmax*Vjpa1)/Jmax;
	Tjb2 = Tjb*Tjb;
	Tjb3 = Tjb*Tjb2;
	Aux  = Jmax*Vjpa1;
	Aux  = Aux*Aux*Aux;
	Aux  = SQRT(Aux);
        Xjpb = Xjpa1+1/2*Vjpa1*Tjb-Jmax*Tjb3/6+(Vjpa1-Jmax*Tjb2/2)*SQRT(Jmax*Vjpa1)/Jmax+Aux/(6*Jmax2);
      }
      if (Xjpb >= GD) { /* IF (Xjpb >= GoalDistance) ==========================================================================(13 i) */
	if (A0==-Amax) { /* IF (A0==-Amax) ====================================================================================(14 i) */
	  Tjpa1 = -A0/Jmax;
	  Ajpa1 = A0 + Jmax*Tjpa1;
	  Vjpa1 = V0 + A0*Tjpa1 + Jmax*Tjpa1*Tjpa1/2;
	  Xjpa1 = X0 + V0*Tjpa1 + A0*Tjpa1*Tjpa1/2 + Jmax*Tjpa1*Tjpa1*Tjpa1/6;
	  if (Xjpa1-GD<= 0.000001 && GD-Xjpa1>=-0.000001) 
	    if (Vjpa1>0) {
	      Tx=0.001;
	      V0=V0+A0*Tx;
	      X0=X0+V0*Tx+A0*Tx*Tx/2;
	    }
	} /* END (A0==-Amax) ==================================================================================================(14 n) */
	Tjpa = (-A0+SQRT(2*A02-4*V0*Jmax+4*Amax2)/2)/Jmax;
	Taca = 0;
	Tjna = 0;
	Tjnb = (A0+Jmax*Tjpa+Amax)/Jmax;
	Tacb = 0;
	Tjpb = Amax/Jmax;
	T->Tjpa = Tjpa;
	T->Taca = Taca;
	T->Tjna = Tjna;
	T->Tvc  = Tvc;
	T->Tjnb = Tjnb;
	T->Tacb = Tacb;
	T->Tjpb = Tjpb;
	if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_OFF ) != TP_OK ) { /* IF (Verify Times) =================(15 i) */
	  XRes = Xjpa1-GD;
	  if (FC.x < GD) { /* IF (FC.x < GD) ==================================================================================(16 i) */
	    FlagCase = 10;
	    Co4= Jmax2/(2*Amax);
	    Co3= Jmax + 2*Jmax*A0/Amax;
	    Co2= 3*A0 + Amax/2 + Jmax*V0/Amax + 5*A02/(2*Amax);
	    Co1= 2*V0 + 2*V0*A0/Amax + A0*Amax/Jmax + 2*A02/Jmax + A03/(Jmax*Amax);
	    Co0= V02/(2*Amax) + XRes + A04/(8*Jmax2*Amax) + V0*A0/Jmax + V0*Amax/(2*Jmax) + A02*Amax/(4*Jmax2) + A03/(3*Jmax2) + A02*V0/(2*Jmax*Amax);
	    VR_Pol.el[0][0] = Co4;
	    VR_Pol.el[0][1] = Co3;
	    VR_Pol.el[0][2] = Co2;
	    VR_Pol.el[0][3] = Co1;
	    VR_Pol.el[0][4] = Co0;
	    if (quarticRSolution(VR_Pol, VR_Sol, TP_OFF) != TP_OK) {
	      fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - quarticSolution 16 i\n");
	      return TP_ERROR;
	    }
	    if(tpSelectTime(VR_Sol,&Tjpa)!=TP_OK) {
	      fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - select Time     11 i\n");
	      return TP_ERROR;
	    }
	    Taca = 0;
	    Tjna = 0;
	    Tjnb = (A0+Jmax*Tjpa+Amax)/Jmax;
	    Tacb = (2*V0*Jmax+4*A0*Tjpa*Jmax+2*Jmax2*Tjpa*Tjpa+A02-2*Amax2)/(2*Jmax*Amax);
	    if (Tacb<0.000001 && Tacb>-0.000001)
	      Tacb=0;
	    Tjpb = Amax/Jmax;
	  } else { /* ELSE (FC.x < GD) ========================================================================================(16 e) */
	    FlagCase = 11;
	    K1=SQRT(2)/12;
	    K2=2*SQRT(3);
	    S0=(2*Jmax*V0-A02);
	    if (S0==0)
	      S0=0.00001;
	    S1=S0*S0*S0;
	    S5=S1*S0*S0;
	    S2a1=-6*A0*Jmax*V0+3*XRes*Jmax2+2*A03;
	    S2a2=Jmax2*(24*XRes*S2a1+30*V02*(S0+2*Jmax*V0));
	    S2a = 144*XRes*Jmax2*S2a1*(S2a2-A02*(6*Jmax2*V02+A02*(S0+4*Jmax*V0)));
	    S2b1=16*Jmax3*V03*(21*S0-5*Jmax*V0)-18*A04*Jmax*V0*S0+3*A06*(3*S0+4*Jmax*V0);
	    S2b = 36*Jmax2*V02*S2b1;
	    S2c = 3*A010*(S0+10*Jmax*V0);
	    S2d1 =Jmax2*(72*XRes*S2a1+V02*(84*S0+32*Jmax*V0));
	    S2d2=Jmax2*(24*XRes*S2a1+36*V02*S0);
	    S2d3=(S2d2+A04*(S0+4*Jmax*V0));
	    S2d = (S2d1+A04*(S0+4*Jmax*V0))*S2d3*S2d3*S2d3;
	    S2 =(S2a + S2b + S2c + K2*SQRT(S2d))*S1*S1;
	    S2p3 = cbrt(S2);
	    S2p6 = cbrt(S2*S2);
	    S3a1=Jmax2*(96*XRes*S2a1+V02*(108*S0+48*Jmax*V0))+A04*(S0+4*Jmax*V0);
	    S3a = 2*K3p6*S5*S3a1;
	    S3b1=12*Jmax2*(XRes*S2a1-V02*(6*S0-10*Jmax*V0))-7*A04*(S0+4*Jmax*V0);
	    S3b = -4*S2p3*S0*S3b1;
	    S3=-(S3a + S3b -2*K3p3*S2p6)/(S2p3*S1);
	    S3p5=SQRT(S3);
	    P1= -(3*Jmax2*XRes+3*Jmax*V0*A0-2*A03)/(3*S0);
	    P2 = K3p6*S3p5*S5*S3a1;
	    P3a=12*Jmax2*(2*XRes*S2a1+V02*(9*S0-12*A02))+19*A04*(S0+4*Jmax*V0);
	    P3 = 8*S2p3*(3*XRes*Jmax2-3*A0*V0*Jmax+A03)*P3a;
	    P4 = 4*S3p5*S2p3*S0*S3b1;
	    P5 = 1/(S3p5*S2p3*S1);
	    ITjpa1 = K1*SQRT((P2 - P3 + P4 - K3p3*S2p6*S3p5)*P5);
	    ITjpa2 = K1*SQRT((P2 + P3 + P4 - K3p3*S2p6*S3p5)*P5);
	    VR_Sol.el[0][0] = (P1+S3p5/12+ITjpa1)/Jmax;
	    VR_Sol.el[0][1] = (P1+S3p5/12-ITjpa1)/Jmax;
	    VR_Sol.el[0][2] = (P1-S3p5/12+ITjpa2)/Jmax;
	    VR_Sol.el[0][3] = (P1-S3p5/12-ITjpa2)/Jmax;
	    if(tpSelectTime(VR_Sol,&Tjpa)!=TP_OK) {
	      fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - select Time     16 e\n");
	      return TP_ERROR;
	    }
	    Vr = V0+A0*Tjpa+Jmax*Tjpa*Tjpa/2;
	    Ar =A0+Jmax*Tjpa;
	    Taca = 0;
	    Tjna = 0;
	    Tjnb = (Ar+SQRT(2*Ar*Ar+4*Vr*Jmax)/2)/Jmax;
	    Tacb = 0;
	    Tjpb = SQRT(2*Ar*Ar+4*Vr*Jmax)/(2*Jmax);
	    T->Tjpa = Tjpa;
	    T->Taca = Taca;
	    T->Tjna = Tjna;
	    T->Tvc  = Tvc;
	    T->Tjnb = Tjnb;
	    T->Tacb = Tacb;
	    T->Tjpb = Tjpb;
	    if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_OFF) != TP_OK) { /* IF (Verify Times) ===============(17 i) */
	      FlagCase=12;
	      Co4= Jmax2/(2*Amax);
	      Co3= Jmax + 2*Jmax*A0/Amax;
	      Co2= 3*A0 + Amax/2 + Jmax*V0/Amax + 5*A02/(2*Amax);
	      Co1= 2*V0 + 2*V0*A0/Amax + A0*Amax/Jmax + 2*A02/Jmax + A03/(Jmax*Amax);
	      Co0= V02/(2*Amax) + XRes + A04/(8*Jmax2*Amax) + A0*V0/Jmax + V0*Amax/(2*Jmax) + A02*Amax/(4*Jmax2) + A03/(3*Jmax2) + A02*V0/(2*Jmax*Amax);
	      VR_Pol.el[0][0] = Co4;
	      VR_Pol.el[0][1] = Co3;
	      VR_Pol.el[0][2] = Co2;
	      VR_Pol.el[0][3] = Co1;
	      VR_Pol.el[0][4] = Co0;
	      if (quarticRSolution(VR_Pol, VR_Sol, TP_OFF) != TP_OK) {
		fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - quarticSolution 17 e\n");
		return TP_ERROR;
	      }
	      if(tpSelectTime(VR_Sol,&Tjpa)!=TP_OK) {
		fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - select Time     17 e\n");
		return TP_ERROR;
	      }
/* 	      fprintf(stderr,"TJPA %f \n\n",Tjpa); */
	      Taca = 0;
	      Tjna = 0;
	      Tjnb = (A0+Jmax*Tjpa+Amax)/Jmax;
	      Tacb = (2*V0*Jmax+4*A0*Tjpa*Jmax+2*Jmax2*Tjpa*Tjpa+A02-2*Amax2)/(2*Jmax*Amax);
	      if (Tacb<0.000001 && Tacb>-0.000001)
		Tacb=0;
	      Tjpb = Amax/Jmax;
	    } /* END (Verify Times) ===========================================================================================(17 n) */ 
	  } /* END (FC.x < GD) ================================================================================================(16 n) */
	} /* END (Verify Times) ===============================================================================================(15 n) */
      } /* END (Xjpb >= GoalDistance) =========================================================================================(13 n) */
      if (Xjpb < GD) { /* IF (Xjpb < GoalDistance) ============================================================================(18 i) */
        XRes = Xjpa1-GD;
        if (Vjpa1 != 0) { /* IF (Vjpa1 != 0) ================================================================================(19 i) */
	  Vjpa12 = Vjpa1*Vjpa1;
	  Vjpa13 = Vjpa1*Vjpa12;
	  Vjpa15 = Vjpa12*Vjpa13;
	  XRes2  = XRes*XRes;
	  FS1a   = Jmax*XRes2+Vjpa13;
          FS1= Vjpa13*Jmax*XRes2-(Vjpa13+Jmax*XRes2)*(54*Jmax*XRes2+37*Vjpa13)+6*SQRT(3)*SQRT((27*Jmax*XRes2+25*Vjpa13)*FS1a*FS1a*FS1a);
	  FS1p3 = cbrt(FS1);
	  FS1p6 = cbrt(FS1*FS1);
	  FS2= -3*Jmax*(-3*XRes2*Jmax*FS1p3-2*Vjpa13*FS1p3-FS1p6*Vjpa1+11*Vjpa15+12*Vjpa12*XRes2*Jmax)/FS1p3;
	  FS2p5 = SQRT(FS2);
	  FS1FS2 = FS1p3*FS2p5;
	  P1 = XRes*Jmax/(2*Vjpa1);
	  P2 = FS2p5/(6*Vjpa1);
	  P3 = 6*XRes2*Jmax*FS1FS2+4*Vjpa13*FS1FS2-FS2p5*FS1p6*Vjpa1+11*FS2p5*Vjpa15+12*FS2p5*Vjpa12*XRes2*Jmax;
	  P4 = 18*XRes*Jmax*FS1p3*Vjpa13;
	  P5 = 18*XRes2*XRes*Jmax2*FS1p3;
	  P6 = FS1p3*FS2p5;
	  Aux = P1+P2+1/6*SQRT(3)/Vjpa1*SQRT(Jmax*(P3+P4+P5)/P6);
	  VR_Sol.el[0][0] = Aux*Aux/Jmax;
	  Aux = P1+P2-1/6*SQRT(3)/Vjpa1*SQRT(Jmax*(P3+P4+P5)/P6);
	  VR_Sol.el[0][1] = Aux*Aux/Jmax;
	  Aux = P1-P2+1/6*SQRT(3)/Vjpa1*SQRT(Jmax*(P3-P4-P5)/P6);
	  VR_Sol.el[0][2] = Aux*Aux/Jmax;
	  Aux = P1-P2-1/6*SQRT(3)/Vjpa1*SQRT(Jmax*(P3-P4-P5)/P6);
	  VR_Sol.el[0][3] = Aux*Aux/Jmax;
	  if(tpSelectTime(VR_Sol,&Vr)!=TP_OK) {
	    fprintf(stderr,"tpJerkProfile ESD - TP_ERROR - select Vr     19 i\n");
	    return TP_ERROR;
	  }
	  Tja = SQRT(-Jmax*(Vjpa1-Vr))/Jmax;
	  Tjb = SQRT(Jmax*Vr)/Jmax;
	  Tjpa = Tjpa1 + Tja;
	  if (Tjpa<SAMPLING_TIME) {
	    Tjpa = 0.0;
	  }
	  Taca = 0;
	  Tjna = Tja;
	  Tacb = 0;
        } else { /* ELSE ( Vjpa1 != 0 ) =====================================================================================(19 e) */
	  Vr = cbrt((-4*XRes*Jmax2)*(-4*XRes*Jmax2))/(4*Jmax);
	  Tja = SQRT(Jmax*Vr)/Jmax;
	  Tjb = Tja;
	  Tjpa = Tjpa1 + Tja;
	  if (Tjpa<SAMPLING_TIME) {
	    Tjpa = 0.0;
	  }
	  Taca = 0;
	  Tjna = Tja;
	  Tacb = 0;
        } /* END ( Vjpa1 != 0 ) =============================================================================================(19 n) */
	Tjnb = Tjb;
	Tjpb = Tjb;
      } /* END (Xjpb < GoalDistance) ==========================================================================================(18 n) */
      T->Tjpa = Tjpa;
      T->Taca = Taca;
      T->Tjna = Tjna;
      T->Tvc  = Tvc;
      T->Tjnb = Tjnb;
      T->Tacb = Tacb;
      T->Tjpb = Tjpb;
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_OFF) != TP_OK) {
	if (tpStopPosition(IC, Limits, &Aux, T)!= TP_OK) {
	  fprintf(stdout," CASE _NO_ YET Stop Position Determination \n");
	  return TP_ERROR;
	}
	if (tpVerifyTimes(DISTANCE_TOLERANCE, GD, Jerks, IC, Dir, *T, &FC, &Acc, &Vel, &Pos, TP_ON) != TP_OK) {
	  fprintf(stderr, "FC %d Dir %d \n",FlagCase, Dir);
	  fprintf(stderr, "Temps %f %f %f %f %f %f %f \n",Tjpa, Taca,Tjna,Tvc,Tjnb,Tacb,Tjpb);
	  fprintf(stderr, " tp Jerk Profile ESD -  TP_ERROR - Can not find Solution \n");
	  fprintf(stderr, " A0 = %f , V0 = %f, X0 = %f, GD = %f \n",IC.a,IC.v,IC.x,GD);
	  return TP_ERROR;
	}
      }
    } /* END (Verify Times) ===================================================================================================(12 n) */

    freemat(VR_Pol);
    freemat(VR_Sol);
    return TP_OK;
}

/* Return Times Structure for Jerk Profile Extention with direction changement
   tpJerkProfileEChD(double GD, TP_LIMITS Limits, TP_COND IC, int Dir, TP_TIMES *T)
    
   Inputs:   GD         Goal Distance
             IC         Initial Conditions  A0, V0, X0
	     Dir        Direction
		  
   Outputs:  T          Times  = Tjnb, Tacb, Tjpb  -------> First Direction  ---> To Stop Condition
             Times  = Tjpa, Taca, Tvc   -------> Second Direction ---> To Position
             Time   = Tjna = 0
    
   Return:   TP_OK
             TP_ERROR      - JerkProfile TP_ERROR      
*/
TP_STATUS tpJerkProfileEChD(double GD, TP_LIMITS Limits, TP_COND IC, int Dir, TP_TIMES *T)
{
  int auxDir = -Dir;
  double auxDist;
  double stopPosition, finalPosition;
  double Jmax = Limits.maxJerk;
  double Amax = Limits.maxAcc;
  double Vmax = Limits.maxVel;
  double Tjpa = 0.0;
  double Taca = 0.0;
  double Tjna = 0.0;
  double Tvc  = 0.0;
  double Tjnb = 0.0;
  double Tacb = 0.0;
  double Tjpb = 0.0;
  double A0, V0, X0;
  double Vc;
  double Jmax2 = Jmax * Jmax; 
  double Amax2 = Amax * Amax;
  double A02, A03, A04;
  double V02;

  /* Positive Deplacement conditions */
  if (IC.v<0) 
    if (auxDir > 0) {
      A0 = -auxDir*IC.a;
      V0 = -auxDir*IC.v;
    } else {
      A0 = auxDir*IC.a;
      V0 = auxDir*IC.v;
    }
  else {
    A0 = IC.a;
    V0 = IC.v;
  }
  
  X0 = IC.x;
  
  A02 = A0*A0;
  A03 = A0*A02;
  A04 = A0*A03;
  
  V02 = V0*V0;
  /* Control Velocity  = 2*Vj*/
  Vc = Amax2/Jmax;
 
  /* To Stop Position */
  if ( ABS(V0) > Vc ) {
    Tjnb = (A0 + Amax)/Jmax;
    if (Tjnb>-SAMPLING_TIME && Tjnb<0) {
      Tjnb= 0.0;
    }
    Tjpb = Amax/Jmax;
    Tacb = (V0*Jmax+0.5*A02-Amax2)/(Jmax*Amax);
    auxDist = V0*A0/Jmax+V0*Amax/(2*Jmax)+A03/(3*Jmax2)+A02*Amax/(4*Jmax2)+V02/(2*Amax)+V0*A02/(2*Jmax*Amax)+A04/(8*Jmax2*Amax);
  }
  else {
/*     Tacb = 0; */
/*     Tjnb = (A0+SQRT(2*A02+4*V0*Jmax)/2)/Jmax; */
/*     if (Tjnb>-SAMPLING_TIME && Tjnb<0) { */
/*       Tjnb= 0.0; */
/*     } */
/*     Tjpb = (-A0+Jmax*Tjnb)/Jmax; */
/*     auxDist = (12*V0*Jmax*A0+(6*V0*Jmax+3*A02)*SQRT(2*A02+4*V0*Jmax)+4*A03)/(12*Jmax2); */
/*   } */
/***/
    Tacb = 0;
    Tjnb = (A0+SQRT(2*A02+4*V0*Jmax)/2)/Jmax;
    Tjpb = (-A0+Jmax*Tjnb)/Jmax;
    auxDist = (12*V0*Jmax*A0+(6*V0*Jmax+3*A02)*SQRT(2*A02+4*V0*Jmax)+4*A03)/(12*Jmax2);
    if (Tjnb<0) {
      Tjnb = 0.0;
      Tacb = -(2*V0*Jmax-A02)/(2*Jmax*A0);
      if (Tacb<0) {
	Tacb = 0.0;
      }
      Tjpb    =-A0/Jmax;
      auxDist = -V02/(2*A0)-A03/(24*Jmax2);
    }
  }
/***/
  stopPosition = X0 + auxDir*auxDist;
  
  /* To Final Position */
  finalPosition = X0 + Dir*GD;
  auxDist = ABS(finalPosition - stopPosition);

  if (tpJerkProfile(auxDist, Jmax, Amax, Vmax, &Tjpa, &Taca, &Tvc, &Tjna) != TP_OK) {
    fprintf(stderr, "tpJerkProfileChD - TP_ERROR - tpJerkProfile \n");
    return TP_ERROR;
  }

  T->Tjpa = Tjpa;
  T->Taca = Taca;
  T->Tjna = 0.0;
  T->Tvc  = Tvc;
  T->Tjnb = Tjnb;
  T->Tacb = Tacb;
  T->Tjpb = Tjpb;  
  
  return TP_OK;
}

/* Jerk Profile Extension in the same direction for fixed times  
   tpJerksPESD(double GD, TP_COND IC, TP_TIMES T, int Dir, TP_LIMITS Limits, double TotalTime, dmat Acc, dmat Vel, dmat Pos);
 
   Inputs:   GD         Goal Distance
             IC         Initial Conditions  A0, V0, X0
	     Dir        Direction
	     T          Times  = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
	     Limits     Limits = maxJerk, maxAcc, maxVel
	     TotalTime  Trajectory Total Time

   Outputs:  Acc        Acceleration Vector
             Vel        Velocity Vector
	     Pos        Position Vector
    
   Return:   TP_OK
             TP_ERROR      - Total Time                          - Max Jerk Function
	                   - Jerk Profile ESD
			   - Js Definition
*/
TP_STATUS tpJerksPESD(double GD, TP_COND IC, TP_TIMES T, int Dir, TP_LIMITS Limits, double TotalTime, TP_VROW *Acc, TP_VROW *Vel, TP_VROW *Pos, int kBase, dmat Js)
{
  double A0, V0, X0;
  double Jmax, Amax, Vmax;
  double DA0, DV0, DX0;
  double Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb;
  double Tjpa2, Tjpa3, Tjna2, Taca2, Tjnb2, Tacb2, Tjpb2;
  double A02, A03, A04, A05,A06, V02, V03;
  double Amax2, Amax3, Amax4, Jmax2, Jmax3;
  double auxTT, auxTA, auxTD, auxTT2;
  double Ajna, Aaca, Vjna, Vaca, Vc, Vjp;
  double aux1, aux2, aux3, aux4;
  double TimeFactorA, TimeFactorD;
  double Vpa, Vpb, Vpc, Vp;
  double Ajpa;
  double J1, J2, J3, J4;
  TP_TIMES auxT, TM,AccV,VelV,PosV;
  int FlagCase;
  TP_JERKS Jerks;
  TP_COND FC;
  double maxJerk;
  int errno, NOE;
  dmat VR_Pol;
  dmat VR_Sol;

  A0 = IC.a;
  if (A0> - ACCELERATION_ZERO && A0 < ACCELERATION_ZERO)
    A0=0;
  V0 = IC.v;
  X0 = IC.x;

  DA0 = Dir*A0;
  DV0 = Dir*V0;
  DX0 = Dir*X0;

  Tjpa = T.Tjpa;
  Taca = T.Taca;
  Tjna = T.Tjna;
  Tvc  = T.Tvc;
  Tjnb = T.Tjnb;
  Tacb = T.Tacb;
  Tjpb = T.Tjpb;

  auxTT = Tjpa + Taca + Tjna + Tvc + Tjnb + Tacb + Tjpb;
  if (ABS(auxTT - TotalTime)>= 0.00001) {
    fprintf(stderr, " tpJerksPESD - TP_ERROR - Total Time \n");
    return TP_ERROR;
  }
  
  FlagCase = 0;
  
  if (auxTT != 0.0) { /* IF (auxTT != 0.0) --------------------------------------------------------------------------------------- 0 I */
    if (Tjpa != 0.0 && Tjna != 0.0) { /* IF (Tjpa != 0.0 && Tjna ~= 0.0) --------------------------------------------------------- 1 I */ 
      Tjpa2 = Tjpa*Tjpa;
      Tjpa3 = Tjpa*Tjpa2;
      Taca2 = Taca*Taca;
      Tjna2 = Tjna*Tjna;
      Tjnb2 = Tjnb*Tjnb;
      Tacb2 = Tacb*Tacb;
      Tjpb2 = Tjpb*Tjpb;
      Vpa=4*DV0*Tjpa2+12*DV0*Tjpa*Taca+6*DV0*Tjpa*Tjna+DA0*Tjpa3+4*DA0*Tjpa2*Taca+2*DA0*Tjpa2*Tjna-6*GD*Tjpa-12*GD*Taca-6*GD*Tjna+6*DV0*Taca2+6*Taca*DV0*Tjna+3*DA0*Tjpa*Taca2+3*Taca*DA0*Tjpa*Tjna+DA0*Tjpa*Tjna2+2*DV0*Tjna2;
      Vpb=Tjnb+2*Tacb+Tjpb;
      Vpc=3*Tacb*Tjpa*Tjpb+6*Tacb*Taca*Tjpb+3*Tacb*Tjna*Tjpb+6*Tjnb*Taca*Tjpb+6*Tjnb*Tjna*Tacb+3*Tjnb*Tjna*Tjpb+4*Tjna2*Tacb+6*Tjnb*Tjpa*Tacb+3*Tjnb*Tjpa*Tjpb+12*Tjnb*Taca*Tacb+3*Tacb2*Tjna+6*Tacb2*Taca+3*Tacb2*Tjpa+2*Tjnb2*Tjna+2*Tjnb2*Tjpa+4*Tjnb2*Taca+Tjpb2*Tjna+2*Tjna2*Tjnb+2*Tjpb2*Taca+Tjpb2*Tjpa+Tjpa2*Tjnb+3*Taca2*Tjpb+6*Tvc*Tjna*Tacb+3*Tvc*Tjna*Tjpb+2*Tjna2*Tjpb+3*Tvc*Tjpa*Tjnb+6*Tvc*Tjpa*Tacb+3*Tvc*Tjpa*Tjpb+6*Tvc*Taca*Tjnb+12*Tvc*Taca*Tacb+6*Tvc*Taca*Tjpb+3*Tvc*Tjna*Tjnb+3*Tjpa*Tjna*Tjnb+6*Tjpa*Tjna*Tacb+3*Tjpa*Taca*Tjnb+6*Tjpa*Taca*Tacb+6*Taca*Tjna*Tjpb+3*Taca2*Tjnb+3*Tjpa*Taca*Tjpb+6*Taca2*Tacb+6*Taca*Tjna*Tjnb+12*Taca*Tjna*Tacb+3*Tjpa*Tjna*Tjpb+Tjpa2*Tjpb+2*Tjpa2*Tacb;
      Vp = -Vpa*Vpb/(2*Vpc);
	 
      J1 = -(2*DV0+2*DA0*Tjpa+2*DA0*Taca+DA0*Tjna-2*Vp)/(Tjpa*(Tjpa+2*Taca+Tjna));
      J2 = (DA0+J1*Tjpa)/Tjna;
      J3 = 2*Vp/(Tjnb*(Tjnb+2*Tacb+Tjpb));
      J4 = J3*Tjnb/Tjpb;
      Ajpa = DA0+Tjpa*Dir*J1;
    } else { /* ELSE (Tjpa != 0.0 && Tjna ~= 0.0) -------------------------------------------------------------------------------- 1 E */
      TimeFactorA = Tjpa + Taca + Tjna + Tvc;
      if (tpJerkProfileESD(GD, Limits, IC, Dir, &auxT) != TP_OK) {
	fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - Jerk Profile ESD 1 \n");
	freemat(Js);
	return TP_ERROR;
      }
      Tjpa = auxT.Tjpa;
      Taca = auxT.Taca;
      Tjna = auxT.Tjna;
      Tvc  = auxT.Tvc;
      auxTA = Tjpa + Taca + Tjna + Tvc;
      if (auxTA != 0.0) {
	Tjpa = Tjpa*TimeFactorA/auxTA;
	Taca = Taca*TimeFactorA/auxTA;
	Tjna = Tjna*TimeFactorA/auxTA;
	Tvc  =  Tvc*TimeFactorA/auxTA;
      } else {
	Tjpa = TimeFactorA/2;
	Tjna = Tjpa;
      }
      Tjpa2 = Tjpa*Tjpa;
      Tjpa3 = Tjpa*Tjpa2;
      Taca2 = Taca*Taca;
      Tjna2 = Tjna*Tjna;
      Tjnb2 = Tjnb*Tjnb;
      Tacb2 = Tacb*Tacb;
      Tjpb2 = Tjpb*Tjpb;
      Vpa=4*DV0*Tjpa2+12*DV0*Tjpa*Taca+6*DV0*Tjpa*Tjna+DA0*Tjpa3+4*DA0*Tjpa2*Taca+2*DA0*Tjpa2*Tjna-6*GD*Tjpa-12*GD*Taca-6*GD*Tjna+6*DV0*Taca2+6*Taca*DV0*Tjna+3*DA0*Tjpa*Taca2+3*Taca*DA0*Tjpa*Tjna+DA0*Tjpa*Tjna2+2*DV0*Tjna2;
      Vpb=Tjnb+2*Tacb+Tjpb;
      Vpc=3*Tacb*Tjpa*Tjpb+6*Tacb*Taca*Tjpb+3*Tacb*Tjna*Tjpb+6*Tjnb*Taca*Tjpb+6*Tjnb*Tjna*Tacb+3*Tjnb*Tjna*Tjpb+4*Tjna2*Tacb+6*Tjnb*Tjpa*Tacb+3*Tjnb*Tjpa*Tjpb+12*Tjnb*Taca*Tacb+3*Tacb2*Tjna+6*Tacb2*Taca+3*Tacb2*Tjpa+2*Tjnb2*Tjna+2*Tjnb2*Tjpa+4*Tjnb2*Taca+Tjpb2*Tjna+2*Tjna2*Tjnb+2*Tjpb2*Taca+Tjpb2*Tjpa+Tjpa2*Tjnb+3*Taca2*Tjpb+6*Tvc*Tjna*Tacb+3*Tvc*Tjna*Tjpb+2*Tjna2*Tjpb+3*Tvc*Tjpa*Tjnb+6*Tvc*Tjpa*Tacb+3*Tvc*Tjpa*Tjpb+6*Tvc*Taca*Tjnb+12*Tvc*Taca*Tacb+6*Tvc*Taca*Tjpb+3*Tvc*Tjna*Tjnb+3*Tjpa*Tjna*Tjnb+6*Tjpa*Tjna*Tacb+3*Tjpa*Taca*Tjnb+6*Tjpa*Taca*Tacb+6*Taca*Tjna*Tjpb+3*Taca2*Tjnb+3*Tjpa*Taca*Tjpb+6*Taca2*Tacb+6*Taca*Tjna*Tjnb+12*Taca*Tjna*Tacb+3*Tjpa*Tjna*Tjpb+Tjpa2*Tjpb+2*Tjpa2*Tacb;
      if (Vpc!=0.0) {
	Vp = -Vpa*Vpb/(2*Vpc);
	J1 = -(2*DV0+2*DA0*Tjpa+2*DA0*Taca+DA0*Tjna-2*Vp)/(Tjpa*(Tjpa+2*Taca+Tjna));
	J2 = (DA0+J1*Tjpa)/Tjna;
	J3 = 2*Vp/(Tjnb*(Tjnb+2*Tacb+Tjpb));
	J4 = J3*Tjnb/Tjpb;
      } else {
	Vp=DV0;
	J1=0;
	J2=0;
	J3 = 2*Vp/(Tjnb*(Tjnb+2*Tacb+Tjpb));
	J4 = J3*Tjnb/Tjpb;
      }
    } /* END (Tjpa != 0.0 && Tjna ~= 0.0) ---------------------------------------------------------------------------------------- 1 N */ 
    Js.el[0][0] = ABS(J1);
    Js.el[0][1] = ABS(J2);
    Js.el[0][2] = ABS(J3);
    Js.el[0][3] = ABS(J4);
    
    if (maxVect(Js, &maxJerk)) {
      fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - maxJerk 1 \n");
      return TP_ERROR;
    }
    
    Jerks.sel = 4;
    Jerks.J1 = J1;
    Jerks.J2 = J2;
    Jerks.J3 = J3;
    Jerks.J4 = J4;

    auxT.Tjpa = Tjpa;
    auxT.Taca = Taca;
    auxT.Tjna = Tjna;
    auxT.Tvc  = Tvc;
    auxT.Tjnb = Tjnb;
    auxT.Tacb = Tacb;
    auxT.Tjpb = Tjpb;
    if (tpVerifyTimes(DISTANCE_TOLERANCE, GD,Jerks,IC,Dir,auxT,&FC,&AccV,&VelV,&PosV,TP_OFF)!=TP_OK || (maxJerk-Limits.maxJerk)>0.00001) {/* IF (tpVerifyTimes!=TP_OK || maxJerk>Jmax) -- 2 I */
      TimeFactorA = T.Tjpa + T.Taca + T.Tjna + T.Tvc;
      if (tpJerkProfileESD(GD, Limits, IC, Dir, &auxT) != TP_OK) {
	fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - Jerk Profile ESD 2 \n");
	return TP_ERROR;
      }
      Tjpa = auxT.Tjpa;
      Taca = auxT.Taca;
      Tjna = auxT.Tjna;
      Tvc  = auxT.Tvc;
      auxTA = Tjpa + Taca + Tjna + Tvc;
      if (auxTA != 0.0) {
	Tjpa = Tjpa*TimeFactorA/auxTA;
	Taca = Taca*TimeFactorA/auxTA;
	Tjna = Tjna*TimeFactorA/auxTA;
	Tvc  =  Tvc*TimeFactorA/auxTA;
      } else {
	Tjpa = TimeFactorA/2;
	Tjna = Tjpa;
      }
      Tjpa2 = Tjpa*Tjpa;
      Tjpa3 = Tjpa*Tjpa2;
      Taca2 = Taca*Taca;
      Tjna2 = Tjna*Tjna;
      Tjnb2 = Tjnb*Tjnb;
      Tacb2 = Tacb*Tacb;
      Tjpb2 = Tjpb*Tjpb;
      Vpa=4*DV0*Tjpa2+12*DV0*Tjpa*Taca+6*DV0*Tjpa*Tjna+DA0*Tjpa3+4*DA0*Tjpa2*Taca+2*DA0*Tjpa2*Tjna-6*GD*Tjpa-12*GD*Taca-6*GD*Tjna+6*DV0*Taca2+6*Taca*DV0*Tjna+3*DA0*Tjpa*Taca2+3*Taca*DA0*Tjpa*Tjna+DA0*Tjpa*Tjna2+2*DV0*Tjna2;
      Vpb=Tjnb+2*Tacb+Tjpb;
      Vpc=3*Tacb*Tjpa*Tjpb+6*Tacb*Taca*Tjpb+3*Tacb*Tjna*Tjpb+6*Tjnb*Taca*Tjpb+6*Tjnb*Tjna*Tacb+3*Tjnb*Tjna*Tjpb+4*Tjna2*Tacb+6*Tjnb*Tjpa*Tacb+3*Tjnb*Tjpa*Tjpb+12*Tjnb*Taca*Tacb+3*Tacb2*Tjna+6*Tacb2*Taca+3*Tacb2*Tjpa+2*Tjnb2*Tjna+2*Tjnb2*Tjpa+4*Tjnb2*Taca+Tjpb2*Tjna+2*Tjna2*Tjnb+2*Tjpb2*Taca+Tjpb2*Tjpa+Tjpa2*Tjnb+3*Taca2*Tjpb+6*Tvc*Tjna*Tacb+3*Tvc*Tjna*Tjpb+2*Tjna2*Tjpb+3*Tvc*Tjpa*Tjnb+6*Tvc*Tjpa*Tacb+3*Tvc*Tjpa*Tjpb+6*Tvc*Taca*Tjnb+12*Tvc*Taca*Tacb+6*Tvc*Taca*Tjpb+3*Tvc*Tjna*Tjnb+3*Tjpa*Tjna*Tjnb+6*Tjpa*Tjna*Tacb+3*Tjpa*Taca*Tjnb+6*Tjpa*Taca*Tacb+6*Taca*Tjna*Tjpb+3*Taca2*Tjnb+3*Tjpa*Taca*Tjpb+6*Taca2*Tacb+6*Taca*Tjna*Tjnb+12*Taca*Tjna*Tacb+3*Tjpa*Tjna*Tjpb+Tjpa2*Tjpb+2*Tjpa2*Tacb;
      if (Vpc!=0.0 && Tjna != 0.0) {
	Vp = -Vpa*Vpb/(2*Vpc);
	J1 = -(2*DV0+2*DA0*Tjpa+2*DA0*Taca+DA0*Tjna-2*Vp)/(Tjpa*(Tjpa+2*Taca+Tjna));
	if (Tjna != 0.0)
	  J2 = (DA0+J1*Tjpa)/Tjna;
	else
	  J2 = 0;
	J3 = 2*Vp/(Tjnb*(Tjnb+2*Tacb+Tjpb));
	J4 = J3*Tjnb/Tjpb;
      } else {
	Vp=DV0;
	J1=0;
	J2=0;
	J3 = 2*Vp/(Tjnb*(Tjnb+2*Tacb+Tjpb));
	J4 = J3*Tjnb/Tjpb;
      }
      Js.el[0][0] = ABS(J1);
      Js.el[0][1] = ABS(J2);
      Js.el[0][2] = ABS(J3);
      Js.el[0][3] = ABS(J4);
	 
      if (maxVect(Js, &maxJerk)) {
	fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - maxJerk 1 \n");
	return TP_ERROR;
      }
	 
      Jerks.sel = 4;
      Jerks.J1 = J1;
      Jerks.J2 = J2;
      Jerks.J3 = J3;
      Jerks.J4 = J4;
	 
      auxT.Tjpa = Tjpa;
      auxT.Taca = Taca;
      auxT.Tjna = Tjna;
      auxT.Tvc  = Tvc;
      auxT.Tjnb = Tjnb;
      auxT.Tacb = Tacb;
      auxT.Tjpb = Tjpb;
      if (tpVerifyTimes(DISTANCE_TOLERANCE, GD,Jerks,IC,Dir,auxT,&FC,&AccV,&VelV,&PosV,TP_OFF)!=TP_OK || (maxJerk-Limits.maxJerk)>0.00001) {/* IF (tpVerifyTimes!=TP_OK || maxJerk>Jmax) -- 3 I */
	if (tpJerkProfileESD(GD, Limits, IC, Dir, &auxT) != TP_OK) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - Jerk Profile ESD 3 \n");
	  return TP_ERROR;
	}
	Tjpa = auxT.Tjpa;
	Taca = auxT.Taca;
	Tjna = auxT.Tjna;
	Tvc  = auxT.Tvc;
	Tjnb = auxT.Tjnb;
	Tacb = auxT.Tacb;
	Tjpb = auxT.Tjpb;

	auxTA = Tjpa + Taca + Tjna + Tvc;
	auxTD = Tjnb + Tacb + Tjpb;

	TimeFactorA = T.Tjpa + T.Taca + T.Tjna + T.Tvc;
	TimeFactorD = T.Tjnb + T.Tacb + T.Tjpb;

	A0=DA0;
	V0=DV0;
	X0=0;

	A02 = A0*A0;
	A03 = A0*A02;
	A04 = A0*A03;
	A05 = A0*A04;
	A06 = A0*A05;
	V02 = V0*V0;
	V03 = V0*V02;

	Jmax=Limits.maxJerk;
	Amax=Limits.maxAcc;
	Vmax=Limits.maxVel;

	Amax2 = Amax*Amax;
	Amax3 = Amax*Amax2;
	Amax4 = Amax*Amax3;
	Jmax2 = Jmax*Jmax;
	Jmax3 = Jmax*Jmax2;

	auxTT = TimeFactorA + TimeFactorD;
	auxTT2 = auxTT*auxTT;
	J1= Jmax;
	J2= Jmax;

	VR_Pol = newdmat(0,0,0,3,&errno);
	if (errno) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - VR_Pol Definition 1 \n");
	  return TP_ERROR;
	}
	VR_Sol = newdmat(0,0,0,2,&errno);
	if (errno) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - VR_Sol Definition 1 \n");
	  freemat(VR_Pol);
	  return TP_ERROR;
	}
	VR_Pol.el[0][0] = Jmax;
	VR_Pol.el[0][1] = -TimeFactorA*Jmax-Tacb*Jmax/2-Tjpb*Jmax-2*A0;
	VR_Pol.el[0][2] = 2*TimeFactorA*A0+Tacb*A0+2*Tjpb*A0+A02/Jmax;
	VR_Pol.el[0][3] = -TimeFactorA*A02/(2*Jmax)+TimeFactorA*V0-Tacb*A02/(4*Jmax)+Tacb*V0/2-A02*Tjpb/(2*Jmax)+Tjpb*V0-GD- A03/(6*Jmax2)+X0;
	if (cubicRSolution(VR_Pol,VR_Sol,TP_OFF)!=TP_OK) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - cubic Solution 1 \n");
	  freemat(VR_Sol);
	  freemat(VR_Pol);
	  return TP_ERROR;
	}
	if(tpSelectTime(VR_Sol,&Tjna)!=TP_OK) {
	  fprintf(stderr,"tp Jerks EPSD - TP_ERROR - select Tjna     1\n");
	  freemat(VR_Sol);
	  freemat(VR_Pol);
	  return TP_ERROR;
	}
	freemat(VR_Sol);
	freemat(VR_Pol);
	Ajna = A0 - J1*Tjna;
	if (Ajna > -Amax)
	  Taca = 0;
	else {
	  Tjna = (A0+Amax)/Jmax;
	  VR_Sol = newdmat(0,0,0,1,&errno);
	  if (errno) {
	    fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - VR_Sol Definition 2 \n");
	    freemat(Js);
	    return TP_ERROR;
	  }
	  aux1 = 12*Jmax*Amax*TimeFactorA+6*Jmax*Amax*Tacb+12*Jmax*Amax*Tjpb-18*Amax2-12*Amax*A0;
	  aux2 = 72*Amax*GD*Jmax2+36*Jmax2*Amax2*TimeFactorA*TimeFactorA-36*Jmax*Amax3*TimeFactorA+9*Jmax2*Amax2*Tacb*Tacb-18*Jmax*Amax3*Tacb+36*Jmax2*Amax2*Tjpb*Tjpb-36*Jmax*Amax3*Tjpb-72*Amax*X0*Jmax2+9*Amax4+72*Jmax2*Amax2*TimeFactorA*Tjpb-72*Jmax*Amax2*TimeFactorA*A0+36*Jmax2*Amax2*Tacb*Tjpb-36*Jmax*Amax2*Tacb*A0-72*Jmax*Amax2*Tjpb*A0-72*Amax*V0*Jmax2*Tjpb-36*Amax*A02*TimeFactorA*Jmax-36*Amax*V0*Jmax2*Tacb-72*Amax*V0*TimeFactorA*Jmax2-18*Amax*Jmax*A02*Tacb-36*Amax*Jmax*A02*Tjpb+36*Jmax2*Amax2*TimeFactorA*Tacb+36*Amax3*A0+36*Amax2*A02+12*Amax*A03;
	  VR_Sol.el[0][0] = (aux1+2*SQRT(aux2))/(12*Amax*Jmax);
	  VR_Sol.el[0][1] = (aux1-2*SQRT(aux2))/(12*Amax*Jmax);
	  if(tpSelectTime(VR_Sol,&Taca)!=TP_OK) {
	    fprintf(stderr,"tp Jerks EPSD - TP_ERROR - select Taca     1\n");
	    freemat(VR_Sol);
	    return TP_ERROR;
	  }
	  freemat(VR_Sol);
	}
	Tjpa = (-A0+J1*Tjna)/J2;
	Tvc  = TimeFactorA-Tjpa-Taca-Tjna;
	Aaca = A0 - J1*Tjna;
	Vjna = V0 + A0*Tjna - J1*Tjna*Tjna/2;
	Vaca = Vjna + Aaca*Taca;
	Vc   = Vaca + Aaca*Tjpa +   J2*Tjpa*Tjpa/2;
	J3   = Vc/(Tjpb*(Tacb+Tjpb));
	J4   = J3;
	VR_Pol = newdmat(0,0,0,4,&errno);
	if (errno) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - VR_Pol Definition 2 \n");
	  return TP_ERROR;
	}
	VR_Sol = newdmat(0,0,0,3,&errno);
	if (errno) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - VR_Sol Definition 2 \n");
	  freemat(VR_Pol);
	  return TP_ERROR;
	}
	if (Tvc<0) { /* IF (Tvc<0) ------------------------------------------------------------------------------------------------4 I */ 
	  J1 = Jmax;
	  J2 = J1;
	  J3 = J2;
	  J4 = J3;
	  VR_Pol.el[0][0] = -3*Jmax2/(2*Amax);
	  VR_Pol.el[0][1] = 6*Jmax*A0/Amax+Jmax;
	  VR_Pol.el[0][2] = Amax/2+3*Jmax*V0/Amax-15*A02/(2*Amax)-2*A0-auxTT*Jmax;
	  VR_Pol.el[0][3] = -Amax*A0/Jmax+3*A03/(Amax*Jmax)-6*A0*V0/Amax+A02/Jmax+2*auxTT*A0;
	  VR_Pol.el[0][4] = -auxTT*A02/(2*Jmax)+auxTT*V0-GD-Amax*V0/(2*Jmax)+Amax*A02/(4*Jmax2)+3*A02*V0/(2*Amax*Jmax)-3*A04/(8*Amax*Jmax2)-3*V02/(2*Amax)-A03/(6*Jmax2);
	  if (quarticRSolution(VR_Pol,VR_Sol,TP_OFF)!=TP_OK) {
	    fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - quartic Solution 1 \n");
	    freemat(VR_Sol);
	    freemat(VR_Pol);
	    return TP_ERROR;
	  }
	  if(tpSelectTime(VR_Sol,&Tjna)!=TP_OK) {
	    fprintf(stderr,"tp Jerks EPSD - TP_ERROR - select Tjna     1\n");
	    freemat(VR_Sol);
	    freemat(VR_Pol);
	    return TP_ERROR;
	  }
	  Taca = 0;
	  Tjpa = (-A0+Jmax*Tjna)/Jmax;
	  Tjnb = Amax/Jmax;
	  Tjpb = Tjnb;
	  Vc   = V0+2*A0*Tjna-Jmax*Tjna*Tjna-A02/(2*Jmax);
	  Tacb = Vc/Amax-Amax/Jmax;
	  Tvc  = auxTT-Tjna-Taca-Tjpa-Tjnb-Tacb-Tjpb;
	  Vjp  = Amax2/(2*Jmax);
	  if (Vc<2*Vjp) {
	    Taca = 0;
	    Tacb = 0;
	    VR_Pol.el[0][0] = Jmax*(2*Jmax2*auxTT2+16*Jmax*A0*auxTT-6*Jmax*V0+39*A02)/2;
	    VR_Pol.el[0][1] = -(12*Jmax2*A0*auxTT2-6*Jmax2*V0*auxTT+33*Jmax*A02*auxTT+6*GD*Jmax2-36*Jmax*A0*V0+55*A03)/3;
	    VR_Pol.el[0][2] = (24*GD*Jmax3*auxTT-24*Jmax3*V0*auxTT2+60*Jmax2*A02*auxTT2-48*Jmax2*A0*V0*auxTT+76*A03*auxTT*Jmax+48*GD*Jmax2*A0+36*Jmax2*V02+101*A04-180*A02*V0*Jmax)/(12*Jmax);
	    VR_Pol.el[0][3] = -A0*(-24*Jmax3*V0*auxTT2+12*Jmax2*A02*auxTT2+24*GD*Jmax3*auxTT+10*A03*auxTT*Jmax-12*Jmax2*A0*V0*auxTT+12*GD*Jmax2*A0-36*A02*V0*Jmax+11*A04+36*Jmax2*V02)/(6*Jmax2);
	    VR_Pol.el[0][4] = Jmax*V02*auxTT2+A04*auxTT2/(4*Jmax)-A02*V0*auxTT2-2*GD*Jmax*V0*auxTT+GD*A02*auxTT-A03*V0*auxTT/(3*Jmax)+A05*auxTT/(6*Jmax2)+GD*GD*Jmax+GD*A03/(3*Jmax)+3*A02*V02/(2*Jmax)-3*A04*V0/(4*Jmax2)+11*A06/(72*Jmax3)-V03;
	    if (quarticRSolution(VR_Pol,VR_Sol,TP_OFF)!=TP_OK) {
	      fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - quartic Solution 2 \n");
	      freemat(VR_Sol);
	      freemat(VR_Pol);
	      return TP_ERROR;
	    }
	    if(tpSelectTime(VR_Sol,&Tjna)!=TP_OK) {
	      fprintf(stderr,"tp Jerks EPSD - TP_ERROR - select Tjna 2 \n");
	      freemat(VR_Sol);
	      freemat(VR_Pol);
	      return TP_ERROR;
	    }
	    Tjpa = (-A0+Jmax*Tjna)/Jmax;
	    Vc = V0+2*A0*Tjna-Jmax*Tjna*Tjna-A02/(2*Jmax);
	    Tjnb=SQRT(Vc/Jmax);
	    Tjpb=Tjnb;
	    Tvc = auxTT-Tjna-Taca-Tjpa-Tjnb-Tacb-Tjpb;
	  }
	  if (Tvc<0) {
	    VR_Pol.el[0][0] = -Jmax2/(2*Amax);
	    VR_Pol.el[0][1] = Jmax*(2*A0+Amax)/Amax;
	    VR_Pol.el[0][2] = (-2*auxTT*Jmax*Amax+Amax2+2*V0*Jmax-5*A02-4*Amax*A0)/(2*Amax);
	    VR_Pol.el[0][3] = -A0*(-2*auxTT*Jmax*Amax+Amax2-A02+2*V0*Jmax-Amax*A0)/(Jmax*Amax);
	    VR_Pol.el[0][4] = -A02*auxTT+V0*auxTT/(2*Jmax)-Amax*V0/(2*Jmax)+Amax*A02/(4*Jmax2)+A02*V0/(2*Amax*Jmax)-A04/(8*Amax*Jmax2)-V02/(2*Amax)-A03/(6*Jmax2)+X0;
	    if (quarticRSolution(VR_Pol,VR_Sol,TP_OFF)!=TP_OK) {
	      fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - quartic Solution 3 \n");
	      freemat(VR_Sol);
	      freemat(VR_Pol);
	      return TP_ERROR;
	    }
	    if(tpSelectTime(VR_Sol,&Tjna)!=TP_OK) {
	      fprintf(stderr,"tp Jerks EPSD - TP_ERROR - select Tjna 3 \n");
	      freemat(VR_Sol);
	      freemat(VR_Pol);
	      return TP_ERROR;
	    }
	    Tjpa = (-A0+Jmax*Tjna)/Jmax;
	    Taca = 0;
	    Tjnb = Amax/Jmax;
	    Tjpb = Tjnb;
	    Tacb =-(-2*V0*Jmax-4*A0*Tjna*Jmax+2*Jmax2*Tjna*Tjna+A02+2*Amax2)/(2*Jmax*Amax);
	    Tvc  = auxTT-Tjna-Tjpa-Tjnb-Tacb-Tjpb;
	  }
	  Jerks.sel = 4;
	  Jerks.J1 = -J1;
	  Jerks.J2 = -J2;
	  Jerks.J3 = J3;
	  Jerks.J4 = J4;
		
	  auxT.Tjpa = Tjpa;
	  auxT.Taca = Taca;
	  auxT.Tjna = Tjna;
	  auxT.Tvc  = Tvc;
	  auxT.Tjnb = Tjnb;
	  auxT.Tacb = Tacb;
	  auxT.Tjpb = Tjpb;
	  if (tpVerifyTimes(DISTANCE_TOLERANCE, GD,Jerks,IC,Dir,auxT,&FC,&AccV,&VelV,&PosV,TP_OFF)!=TP_OK) { /* IF (tpVerifyTimes!=TP_OK)----------------------- 5 I */
	    aux1 = (4*Jmax*V0-2*A02);
	    aux2 = SQRT(aux1);
	    aux3 = aux1*aux1;
	    aux4 = aux3*aux2;
	    VR_Pol.el[0][0] = -Jmax2*aux2*(4*A02+aux1)/aux3;
	    VR_Pol.el[0][1] = Jmax*(8*A03*aux1+2*A0*aux3+2*A0*aux3+aux4)/(aux4);
	    VR_Pol.el[0][2] = (8*Jmax*A02*V0*aux1-4*A04*aux1+2*Jmax*V0*aux3-2*Jmax*auxTT*aux4-A02*aux3-8*A02*aux3-4*A0*aux4+aux1*aux3)/(2*aux4);
	    VR_Pol.el[0][3] = -A0*(-2*auxTT*Jmax*aux2-A02-A0*aux2+aux1+2*V0*Jmax)/(Jmax*aux2);
	    VR_Pol.el[0][4] = -A02*auxTT/(2*Jmax)+V0*auxTT-V0*aux2/(2*Jmax)-A03/(6*Jmax2)+A02*aux2/(4*Jmax2)-GD;
	    if (quarticRSolution(VR_Pol,VR_Sol,TP_OFF)!=TP_OK) {
	      fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - quartic Solution 3 \n");
	      freemat(VR_Sol);
	      freemat(VR_Pol);
	      return TP_ERROR;
	    }
	    if(tpSelectTime(VR_Sol,&Tjna)!=TP_OK) {
	      fprintf(stderr,"tp Jerks EPSD - TP_ERROR - select Tjna 3 \n");
	      freemat(VR_Sol);
	      freemat(VR_Pol);
	      return TP_ERROR;
	    }
	    Taca = 0;
	    Tacb = 0;
	    Tjnb =  SQRT(4*V0*Jmax+8*A0*Tjna*Jmax-4*Jmax2*Tjna*Tjna-2*A02)/(2*Jmax);
	    Tjpb = Tjnb;
	    Tjpa = (-A0+Jmax*Tjna)/Jmax;
	    Tvc  = auxTT-Tjna-Taca-Tjpa-Tjnb-Tacb-Tjpb;
	    if (Tvc<0) { /* IF (Tvc < 0) ------------------------------------------------------------------------------------------6 I */
	      J1   = Jmax;
	      J2   = J1;
	      Tjna =  (12*V0*Jmax+12*A0*Jmax*auxTT-6*A02+2*SQRT(36*V02*Jmax2+18*A02*Jmax2*auxTT2-12*A03*Jmax*auxTT+3*A04+36*auxTT2*Jmax3*V0+72*auxTT*Jmax3*X0-72*auxTT*Jmax3*GD-72*A0*X0*Jmax2+72*A0*GD*Jmax2))/(2*Jmax*(6*auxTT*Jmax-6*A0));
	      aux1 = -auxTT*Jmax+2*Jmax*Tjna-A0;
	      aux2 = aux1*aux1;
	      J4 = -2*(-2*V0*Jmax-4*A0*Tjna*Jmax+2*Jmax2*Tjna*Tjna+A02)*Jmax/aux2;
	      J3 = J4;
	      Taca = 0;
	      Tacb = 0;
	      Tjpa = (-A0+Jmax*Tjna)/Jmax;
	      Tjnb = (auxTT-Tjna-Tjpa)/2;
	      Tjpb=Tjnb;
	      Tvc=0;
	      if (J3>Jmax) { /* IF (J3 > Jmax) ------------------------------------------------------------------------------------7 I */
		Tacb=0;
		Tvc=0;
		J1   = Jmax;
		J2   = J1;
		Taca = -(12*GD*Jmax2-3*A02*auxTT*Jmax-6*V0*Jmax2*auxTT-6*Amax2*A0+6*Amax2*auxTT*Jmax-6*A02*Amax-6*V0*A0*Jmax-A03-12*V0*Amax*Jmax-12*X0*Jmax2)/(3*Jmax*(-2*A0*Amax-A02-2*V0*Jmax+2*auxTT*Jmax*Amax));
		Tjna = (A0+Amax)/Jmax;
		Tjpa = Amax/Jmax;
		aux1 = -auxTT*Jmax+A0+2*Amax+Taca*Jmax;
		aux2 = aux1*aux1;
		J4 = -2*(-2*V0*Jmax-A02+2*Amax2+2*Amax*Taca*Jmax)*Jmax/aux2;
		J3 = J4;
		if (Taca<0 || J3>Jmax) { /* IF (Taca < 0) ------------------------------------------------------------------------------------8 I */
		  if (Taca<-2*S_T/S_TD || J3>Jmax) {
		    Tjna = (12*V0*Jmax+12*A0*Jmax*auxTT-6*A02+2*SQRT(36*V02*Jmax2+18*A02*Jmax2*auxTT2-12*A03*Jmax*auxTT+3*A04+36*auxTT2*Jmax3*V0+72*auxTT*Jmax3*X0-72*auxTT*Jmax3*GD-72*A0*X0*Jmax2+72*A0*GD*Jmax2))/(2*Jmax*(6*auxTT*Jmax-6*A0));
		    Tjpa = (-A0+Jmax*Tjna)/Jmax;
		    Taca=0;
		    J3=Jmax;
		    J4=J3;
		  } else {
		    Taca=S_T/S_TD;
		    aux1 = -auxTT*Jmax+A0+2*Amax+Taca*Jmax;
		    aux2 = aux1*aux1;
		    J4 = -2*(-2*V0*Jmax-A02+2*Amax2+2*Amax*Taca*Jmax)*Jmax/aux2;
		    J3 = J4;
		  }
		} /* END (Taca < 0) -----------------------------------------------------------------------------------------------8 N */ 
		Tacb = 0;
		Tjnb = (auxTT-Tjna-Tjpa-Taca)/2;
		Tjpb = Tjnb;
		Tvc  = 0;
	      } /* END (J3 < Jmax) ------------------------------------------------------------------------------------------------7 N */
	    } /* END (Tvc < 0) ----------------------------------------------------------------------------------------------------6 N */
	  } /* END (tpVerifyTimes!=TP_OK)---------------------------------------------------------------------------------------------5 N */
	} /* END (Tvc<0) ----------------------------------------------------------------------------------------------------------4 N */ 
	freemat(VR_Sol);
	freemat(VR_Pol);
	J1=-J1;
	J2=-J2;
	aux1=Tjpa;
	Tjpa=Tjna;
	Tjna=fabs(aux1);
        Js.el[0][0] = ABS(J1);
	Js.el[0][1] = ABS(J2);
	Js.el[0][2] = ABS(J3);
	Js.el[0][3] = ABS(J4);
	if (maxVect(Js, &maxJerk)) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - maxJerk 2 \n");
	  return TP_ERROR;
	}
	/* We have to accept small overpass of Jmax 0.2% */
	if (maxJerk-Jmax>0.002*Jmax) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - Jerk Max Limits 1 \n");
	  return TP_ERROR;
	}
	Jerks.sel = 4;
	Jerks.J1 = J1;
	Jerks.J2 = J2;
	Jerks.J3 = J3;
	Jerks.J4 = J4;
	   
	auxT.Tjpa = Tjpa;
	auxT.Taca = Taca;
	auxT.Tjna = Tjna;
	auxT.Tvc  = Tvc;
	auxT.Tjnb = Tjnb;
	auxT.Tacb = Tacb;
	auxT.Tjpb = Tjpb;
	if (tpVerifyTimes(DISTANCE_TOLERANCE, GD,Jerks,IC,Dir,auxT,&FC,&AccV,&VelV,&PosV,TP_ON)!=TP_OK) {
	  fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - Verify Times \n");
	  fprintf(stderr, "Dir %d \n",Dir);
	  fprintf(stderr, "Temps %f %f %f %f %f %f %f \n",Tjpa, Taca,Tjna,Tvc,Tjnb,Tacb,Tjpb);
	  fprintf(stderr, " A0 = %f , V0 = %f, X0 = %f, GD = %f \n",IC.a,IC.v,IC.x,GD);
	  return TP_ERROR;
	}
      } /* END (tpVerifyTimes!=TP_OK || maxJerk>Jmax) --------------------------------------------------------------------------------3 N */
    } /* END (tpVerifyTimes!=TP_OK || maxJerk>Jmax) ----------------------------------------------------------------------------------2 N */
    if (tpGetMonotonicTimes(auxT,&TM,&NOE)!=TP_OK) {
      fprintf(stderr,"tp Jerks EPSD - TP_ERROR - Get Monotonic Times \n");
      return TP_ERROR;
    }
    if (tpBuildVectorsSD(Dir, Limits, Jerks, IC, TM, AccV, VelV, PosV, Acc, Vel, Pos,kBase)!= TP_OK) {
      fprintf(stderr,"tp Jerks EPSD - TP_ERROR - Build Vectors \n");
      return TP_ERROR;
    }
  } else { /* ELSE (auxTT != 0.0) ------------------------------------------------------------------------------------------------ 0 E */ 
    fprintf(stdout,"NOT YET \n");
  } /* END (auxTT != 0.0) -------------------------------------------------------------------------------------------------------- 0 N */ 
  return TP_OK;
}

/* Jerk Profile Start for fixed times  
   tpJerksPEStr(double GD, TP_CTP_OND IC, TP_TIMES T, int Dir, TP_LIMITS Limits, double TotalTime, dmat Acc, dmat Vel, dmat Pos);
 
   Inputs:   GD         Goal Distance
             IC         Initial Conditions  A0, V0, X0
	     Dir        Direction
	     T          Times  = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
	     Limits     Limits = maxJerk, maxAcc, maxVel
	     TotalTime  Trajectory Total Time

   Outputs:  Acc        Acceleration Vector
             Vel        Velocity Vector
	     Pos        Position Vector
    
   Return:   TP_OK
             TP_ERROR      - Total Time                          - Max Jerk Function
                           - Jerk Profile ESD                    - Get Monotonic times
			   - Js Definition                       - Build Vectors
*/

TP_STATUS tpJerksPEStr(double GD, TP_COND IC, TP_TIMES T, int Dir, TP_LIMITS Limits, double TotalTime, TP_VROW *Acc, TP_VROW *Vel, TP_VROW *Pos, int kBase, dmat Js)
{
  double Jmax;
  double Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb;
  double Tjpa2, Taca2, Tjna2, Tjnb2, Tacb2, Tjpb2;
  double J1, J1d, J2, J3, J4;
  double auxTT;
  TP_TIMES TM, AccV, VelV, PosV;
  TP_JERKS Jerks;
  TP_COND FC;
  double maxJerk;
  int NOE;

  Jmax = Limits.maxJerk;

  Tjpa = T.Tjpa;
  Taca = T.Taca;
  Tjna = T.Tjpa;
  Tvc  = T.Tvc;
  Tjnb = T.Tjnb;
  Tacb = T.Tacb;
  Tjpb = T.Tjpb;

  Tjpa2 = Tjpa*Tjpa;
  Taca2 = Taca*Taca;
  Tjna2 = Tjna*Tjna;
  Tjnb2 = Tjnb*Tjnb;
  Tacb2 = Tacb*Tacb;
  Tjpb2 = Tjpb*Tjpb;

  auxTT = Tjpa + Taca + Tjna + Tvc + Tjnb + Tacb + Tjpb;
  if (ABS(auxTT - TotalTime)>= 0.00001) {
    fprintf(stderr, "tp Jerks PEStr - TP_ERROR - Total Time \n");
    return TP_ERROR;
  }
  
  J1d =3*Tvc*Tjpa*Tjpb+6*Tjnb*Tjna*Tacb+2*Tjpa2*Tacb+Tjpa2*Tjpb+2*Tjna2*Tjnb+4*Tjna2*Tacb+2*Tjna2*Tjpb+2*Tjpa*Tjnb2+2*Tjna*Tjnb2+3*Tjpa*Tacb2+3*Tjna*Tacb2+Tjpa*Tjpb2+6*Tjnb*Tjpa*Tacb+3*Tvc*Tjpa*Tjnb+Tjna*Tjpb2+3*Tacb*Tjna*Tjpb+6*Tvc*Tjpa*Tacb+3*Tvc*Tjna*Tjnb+6*Tvc*Tjna*Tacb+3*Tvc*Tjna*Tjpb+3*Tjnb*Tjpa*Tjpb+3*Tjnb*Tjna*Tjpb+3*Tacb*Tjpa*Tjpb+3*Tjna*Tjpa*Tjnb+6*Tjna*Tjpa*Tacb+3*Tjna*Tjpa*Tjpb+Tjpa2*Tjnb+(3*Tjpb+3*Tjnb+6*Tacb)*Taca2+(6*Tjna*Tjpb+6*Tacb*Tjpb+4*Tjnb2+12*Tvc*Tacb+6*Tvc*Tjpb+6*Tacb2+2*Tjpb2+3*Tjpa*Tjnb+6*Tjnb*Tjpb+12*Tjna*Tacb+6*Tvc*Tjnb+12*Tjnb*Tacb+3*Tjpa*Tjpb+6*Tjpa*Tacb+6*Tjna*Tjnb)*Taca;
  J1 = 6*GD*(Tjnb+2*Tacb+Tjpb)/(Tjpa*J1d);
  J2 = J1*Tjpa/Tjna;
  J3 = (J1*Tjpa2+2*Taca*J1*Tjpa+Tjna*J1*Tjpa)/(Tjnb*(Tjnb+2*Tacb+Tjpb));
  J4 = J3*Tjnb/Tjpb;

  Js.el[0][0] = ABS(J1);
  Js.el[0][1] = ABS(J2);
  Js.el[0][2] = ABS(J3);
  Js.el[0][3] = ABS(J4);

  if (maxVect(Js, &maxJerk)) {
    fprintf(stderr,"tp Jerks PEStr -  TP_ERROR - maxJerk 2 \n");
    return TP_ERROR;
  }
  /* We have to accept small overpass of Jmax 0.2% */
  if (maxJerk-Jmax>0.002*Jmax) {
    fprintf(stderr,"tp Jerks PEStr -  TP_ERROR - Jerk Max Limits 1 \n");
    return TP_ERROR;
  }
  Jerks.sel = 4;
  Jerks.J1 = J1;
  Jerks.J2 = J2;
  Jerks.J3 = J3;
  Jerks.J4 = J4;
  
  if (tpVerifyTimes(DISTANCE_TOLERANCE, GD,Jerks,IC,Dir,T,&FC,&AccV,&VelV,&PosV,TP_ON)!=TP_OK) {
    fprintf(stderr,"tp Jerks PEStr -  TP_ERROR - Verify Times \n");
    return TP_ERROR;
  }

  if (tpGetMonotonicTimes(T,&TM,&NOE)!=TP_OK) {
    fprintf(stderr,"tp Jerks PEStr - TP_ERROR - Get Monotonic Times \n");
    return TP_ERROR;
  }
  if (tpBuildVectorsSD(Dir, Limits, Jerks, IC, TM, AccV, VelV, PosV, Acc, Vel, Pos,kBase)!= TP_OK) {
    fprintf(stderr,"tp Jerks PEStr - TP_ERROR - Build Vectors \n");
    return TP_ERROR;
  }
  
  return TP_OK;
}

/* Jerk Profile Extension with change Direction for fixed times  
   tpJerksPEChD(double GD, TP_COND IC, TP_TIMES T, int Dir, TP_LIMITS Limits, double TotalTime, dmat Acc, dmat Vel, dmat Pos);
 
   Inputs:   GD         Goal Distance
             IC         Initial Conditions  A0, V0, X0
	     Dir        Direction
	     T          Times  = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
	     Limits     Limits = maxJerk, maxAcc, maxVel
	     TotalTime  Trajectory Total Time

   Outputs:  Acc        Acceleration Vector
             Vel        Velocity Vector
	     Pos        Position Vector
    
   Return:   TP_OK
             TP_ERROR      - Total Time                          - Max Jerk Function
	                   - Jerk Profile ESD                    - Get Monotonic times
			   - Js Definition                       - Build Vectors	
*/
TP_STATUS tpJerksPEChD(double GD, TP_COND IC, TP_TIMES T1, TP_TIMES T2, int Dir, TP_LIMITS Limits, double TotalTime, TP_VROW *Acc, TP_VROW *Vel, TP_VROW *Pos, int kBase, dmat Js)
{
  double Jmax;
  double A0, V0,aGD;
  double XTacb, XsTacb, Xjpb;
  double Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb;
  double Tjpa2, Taca2, Tjna2, Tjnb2, Tacb2, Tjpb2;
  double J1, J1d, J2, J3, J4;
  double auxTT;
  TP_TIMES auxT, TM, AccV, VelV, PosV;
  TP_JERKS Jerks;
  TP_COND FC,IC2;
  double maxJerk;
  int NOE1, NOE2, Dir1;
  TP_TIMES auxT2;

  Jmax = Limits.maxJerk;
  Dir1 = -Dir;

  A0 = Dir1*IC.a;
  if (A0> - ACCELERATION_ZERO && A0 < ACCELERATION_ZERO)
    A0=0;
  V0 = Dir1*IC.v;

  auxTT = T1.Tjnb + T1.Tacb+ T1.Tjpb + T2.Tjpa + T2.Taca + T2.Tjna + T2.Tvc + T2.Tjnb + T2.Tacb + T2.Tjpb;
  if (ABS(auxTT - TotalTime)>= 0.00001) {
    fprintf(stderr, " tpJerksPEChD - TP_ERROR - Total Time \n");
    return TP_ERROR;
  }
  
  Tjpa = T1.Tjpa;
  Taca = T1.Taca;
  Tjna = T1.Tjna;
  Tvc  = T1.Tvc;
  Tjnb = T1.Tjnb;
  Tacb = T1.Tacb;
  Tjpb = T1.Tjpb;

  Tjpa2 = Tjpa*Tjpa;
  Taca2 = Taca*Taca;
  Tjna2 = Tjna*Tjna;
  Tjnb2 = Tjnb*Tjnb;
  Tacb2 = Tacb*Tacb;
  Tjpb2 = Tjpb*Tjpb;

  /* FOR FIRST MOUVEMENT - STOPING */
  J3     = (2*V0+2*A0*Tjnb+2*Tacb*A0+Tjpb*A0)/(Tjnb*(Tjnb+2*Tacb+Tjpb));
  J4     = -(A0-J3*Tjnb)/Tjpb;
  XTacb  = Tacb*(12*V0*Tjnb+4*A0*Tjnb2+6*Tacb*V0+6*V0*Tjpb+3*Tacb*A0*Tjnb+3*A0*Tjnb*Tjpb);
  XsTacb = (Tjpb+Tjnb)*(A0*Tjnb2+A0*Tjnb*Tjpb+4*V0*Tjnb+2*V0*Tjpb);
  Xjpb   = (XsTacb + XTacb)/(6*(Tjnb+2*Tacb+Tjpb));

  /* Jmax Limitation */
  if (ABS(J3) > Jmax) {
/*     fprintf(stdout,"J3 %f \n",J3); */
    J3=Jmax;
  }
  
  Js.el[0][0] = 0;
  Js.el[0][1] = 0;
  Js.el[0][2] = ABS(J3);
  Js.el[0][3] = ABS(J4);
  
  if (maxVect(Js, &maxJerk)) {
    fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - maxJerk 1 \n");    
    return TP_ERROR;
  }
  /* We have to accept small overpass of Jmax 0.2% */
  if(maxJerk-Jmax>0.002*Jmax) {
    if (tpStopPosition(IC, Limits, &Xjpb, &auxT2)!=TP_OK) {
      fprintf(stderr,"tp Jerks EPChD - TP_ERROR - Aux Times to Stop Position \n");
      return TP_ERROR;
    }
    fprintf(stdout,"T1 %f %f %f %f %f %f %f \n",Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb,Tjpb);
    fprintf(stdout,"Tjnb %f , Tacb %f , Tjpb %f\n",auxT2.Tjnb,auxT2.Tacb,auxT2.Tjpb);
    fprintf(stdout,"maxJ %f \n",maxJerk);
    fprintf(stderr,"tp Jerks EPChD - TP_ERROR - Out of limits Jerk Stop Motion \n");
    return TP_ERROR;
  }
    
  Jerks.sel = 4;
  Jerks.J1 = J1;
  Jerks.J2 = J2;
  Jerks.J3 = J3;
  Jerks.J4 = J4;
  
  auxT.Tjpa = 0;
  auxT.Taca = 0;
  auxT.Tjna = 0;
  auxT.Tvc  = 0;
  auxT.Tjnb = Tjnb;
  auxT.Tacb = Tacb;
  auxT.Tjpb = Tjpb;

  if (tpVerifyTimes(DISTANCE_TOLERANCE, Xjpb,Jerks,IC,Dir1,auxT,&FC,&AccV,&VelV,&PosV,TP_ON)!=TP_OK) {
    fprintf(stderr," tp Jerks EPChD - TP_ERROR -  Verify Times \n");
    fprintf(stdout," Tjpa = %f, Taca = %f, Tjna = %f, Tvc = %f, Tjnb = %f, Tacb = %f, Tjpb = %f\n",auxT.Tjpa,auxT.Taca,auxT.Tjna,auxT.Tvc,auxT.Tjnb,auxT.Tacb,auxT.Tjpb);
    return TP_ERROR;
  }
  
  if (tpGetMonotonicTimes(auxT,&TM,&NOE1)!=TP_OK) {
    fprintf(stderr,"tp Jerks EPChD - TP_ERROR - Get Monotonic Times \n");
    return TP_ERROR;
  }
  
  NOE1 = NOE1 + 3;

  if (tpBuildVectorsSD(Dir1, Limits, Jerks, IC, TM, AccV, VelV, PosV, Acc, Vel, Pos,kBase)!= TP_OK) {
    fprintf(stderr,"tp Jerks EPChD - ERROR - Build Vectors \n");
    return TP_ERROR;
  }

  /* FOR SECOND MOUVEMENT - TO GOAL DISTANCE */
    
  Tjpa = T2.Tjpa;
  Taca = T2.Taca;
  Tjna = T2.Tjna;
  Tvc  = T2.Tvc;
  Tjnb = T2.Tjnb;
  Tacb = T2.Tacb;
  Tjpb = T2.Tjpb;

  Tjpa2 = Tjpa*Tjpa;
  Taca2 = Taca*Taca;
  Tjna2 = Tjna*Tjna;
  Tjnb2 = Tjnb*Tjnb;
  Tacb2 = Tacb*Tacb;
  Tjpb2 = Tjpb*Tjpb;
  

  aGD = ABS(Dir*GD - Dir1*Xjpb);

  J1d =3*Tvc*Tjpa*Tjpb+6*Tjnb*Tjna*Tacb+2*Tjpa2*Tacb+Tjpa2*Tjpb+2*Tjna2*Tjnb+4*Tjna2*Tacb+2*Tjna2*Tjpb+2*Tjpa*Tjnb2+2*Tjna*Tjnb2+3*Tjpa*Tacb2+3*Tjna*Tacb2+Tjpa*Tjpb2+6*Tjnb*Tjpa*Tacb+3*Tvc*Tjpa*Tjnb+Tjna*Tjpb2+3*Tacb*Tjna*Tjpb+6*Tvc*Tjpa*Tacb+3*Tvc*Tjna*Tjnb+6*Tvc*Tjna*Tacb+3*Tvc*Tjna*Tjpb+3*Tjnb*Tjpa*Tjpb+3*Tjnb*Tjna*Tjpb+3*Tacb*Tjpa*Tjpb+3*Tjna*Tjpa*Tjnb+6*Tjna*Tjpa*Tacb+3*Tjna*Tjpa*Tjpb+Tjpa2*Tjnb+(3*Tjpb+3*Tjnb+6*Tacb)*Taca2+(6*Tjna*Tjpb+6*Tacb*Tjpb+4*Tjnb2+12*Tvc*Tacb+6*Tvc*Tjpb+6*Tacb2+2*Tjpb2+3*Tjpa*Tjnb+6*Tjnb*Tjpb+12*Tjna*Tacb+6*Tvc*Tjnb+12*Tjnb*Tacb+3*Tjpa*Tjpb+6*Tjpa*Tacb+6*Tjna*Tjnb)*Taca;
  J1 = 6*aGD*(Tjnb+2*Tacb+Tjpb)/(Tjpa*J1d);
  if (J1 > Jmax) {
    J1 = Jmax;
  }

  J2 = J1*Tjpa/Tjna;
  J3 = (J1*Tjpa2+2*Taca*J1*Tjpa+Tjna*J1*Tjpa)/(Tjnb*(Tjnb+2*Tacb+Tjpb));
  J4 = J3*Tjnb/Tjpb;
  
  Js.el[0][0] = ABS(J1);
  Js.el[0][1] = ABS(J2);
  Js.el[0][2] = ABS(J3);
  Js.el[0][3] = ABS(J4);
  
  if (maxVect(Js, &maxJerk)) {
    fprintf(stderr,"tp Jerks EPSD -  TP_ERROR - maxJerk 2 \n");
    return TP_ERROR;
  }
  
  Jerks.sel = 4;
  Jerks.J1 = J1;
  Jerks.J2 = J2;
  Jerks.J3 = J3;
  Jerks.J4 = J4;
  
  IC2.a = FC.a;
  IC2.v = FC.v;
  IC2.x = FC.x;
  /* We have to accept small overpass of Jmax 0.2% */
  if (tpVerifyTimes(DISTANCE_TOLERANCE, aGD,Jerks,IC2,Dir,T2,&FC,&AccV,&VelV,&PosV,TP_ON)!=TP_OK || (maxJerk-Jmax)>0.002*Jmax) {
    fprintf(stderr,"Times %f %f %f %f %f %f %f \n",T2.Tjpa,T2.Taca,T2.Tjna,T2.Tvc,T2.Tjnb,T2.Tacb,T2.Tjpb);
    fprintf(stderr,"Jmax %f, Jerk Max %f \n",Jmax, maxJerk);
    fprintf(stderr," tp Jerks EPChD - ERROR -  Verify Times OR maxJerk \n");
    matPrintf(Js);
    return TP_ERROR;
  }
  
  if (tpGetMonotonicTimes(T2,&TM,&NOE2)!=TP_OK) {
    fprintf(stderr,"tp Jerks EPChD - ERROR - Get Monotonic Times \n");
    return TP_ERROR;
  }
  
  kBase = kBase + NOE1;

  if (tpBuildVectorsSD(Dir, Limits, Jerks, IC2, TM, AccV, VelV, PosV, Acc, Vel, Pos, kBase)!= TP_OK) {
    fprintf(stderr,"tp Jerks EPChD - ERROR - Build Vectors \n");
    return TP_ERROR;
  }  

  return TP_OK;
}

/* Jerk Profile Extension Stop for fixed times  
   tpJerksPEStp(double GD, TP_COND IC, TP_TIMES T, int Dir, TP_LIMITS Limits, double TotalTime, dmat Acc, dmat Vel, dmat Pos);

   Inputs:   GD         Goal Distance
             IC         Initial Conditions  A0, V0, X0
	     Dir        Direction
	     T          Times  = Tjpa, Taca, Tjna, Tvc, Tjnb, Tacb, Tjpb
	     Limits     Limits = maxJerk, maxAcc, maxVel
	     TotalTime  Trajectory Total Time

   Outputs:  Acc        Acceleration Vector
             Vel        Velocity Vector
	     Pos        Position Vector
    
   Return:   TP_OK
             TP_ERROR      - Total Time                          - Max Jerk Function
	                   - Jerk Profile ESD                    - Get Monotonic times
			   - Js Definition                       - Build Vectors	
*/
TP_STATUS tpJerksPEStp(double GD, TP_COND IC, TP_TIMES T, int Dir, TP_LIMITS Limits, double TotalTime, TP_VROW *Acc, TP_VROW *Vel, TP_VROW *Pos, int kBase)
{
  double Jmax;
  double A0, V0;
  double Tjnb, Tacb, Tjpb;
  double Tjnb2, Tjnb3;
  double A02, A03, A04, A05, A06;
  double V02, V03, Jmax2, Jmax3, Jmax4, SD, GD2;
  double auxTT;
  TP_TIMES TM, auxT, TAccV, TVelV, TPosV;
  TP_JERKS Jerks;
  TP_COND FC;
  dmat VR_Sol;
  int errno, NOE;

  Jmax = Limits.maxJerk;

  A0 = Dir*IC.a;
  if (A0> - ACCELERATION_ZERO && A0 < ACCELERATION_ZERO)
    A0=0;
  V0 = Dir*IC.v;

  A02 = A0*A0;
  A03 = A0*A02;
  A04 = A0*A03;
  A05 = A0*A04;
  A06 = A0*A05;

  V02 = V0*V0;
  V03 = V0*V02;

  Jmax2 = Jmax*Jmax;
  Jmax3 = Jmax*Jmax2;
  Jmax4 = Jmax*Jmax3;
  
  GD2 = GD*GD;

  auxTT = T.Tjpa + T.Taca+ T.Tjna  + T.Tvc + T.Tjnb + T.Tacb + T.Tjpb;
  if (ABS(auxTT - TotalTime)>= 0.00001) {
    fprintf(stderr, "tp Jerks PEStp - TP_ERROR - Total Time \n");
    fprintf(stderr, "auxTT %g TotalTime%g \n",auxTT,TotalTime);
    return TP_ERROR;
  }

  Tjnb = T.Tjnb;
  Tjnb2 = Tjnb*Tjnb;
  Tjnb3 = Tjnb*Tjnb;
  
  VR_Sol = newdmat(0,0,0,1,&errno);
  if (errno) {
    fprintf(stderr,"tp Jerks PEStp -  TP_ERROR - VR_Sol Definition \n");
    return TP_ERROR;
  }
  auxTT = -2*A06-96*A03*GD*Jmax2+144*GD2*Jmax4-12*V0*Jmax*A04-288*V0*Jmax3*GD*A0-144*V03*Jmax3-72*A02*V02*Jmax2;
  VR_Sol.el[0][0] = (4*A03+24*GD*Jmax2+2*SQRT(auxTT))/(Jmax*2*(12*V0*Jmax+6*A02));
  VR_Sol.el[0][1] = (4*A03+24*GD*Jmax2-2*SQRT(auxTT))/(Jmax*2*(12*V0*Jmax+6*A02));
  if(tpSelectTime(VR_Sol,&Tjnb)!=TP_OK) {
    fprintf(stderr,"tp Jerk PEStp - TP_ERROR - select Time     \n");
    freemat(VR_Sol);
    return TP_ERROR;
  }

  Tjpb = (-A0+Jmax*Tjnb)/Jmax;
  Tacb = -(-2*V0*Jmax-4*A0*Tjnb*Jmax+2*Jmax2*Tjnb*Tjnb+A02)/(2*Jmax*(-A0+Jmax*Tjnb));
  if (Tacb<0) Tacb=0;
  Jerks.sel = 4;
  Jerks.J1 = 0;
  Jerks.J2 = 0;
  Jerks.J3 = Jmax;
  Jerks.J4 = Jmax;
  
  auxT.Tjpa = 0;
  auxT.Taca = 0;
  auxT.Tjna = 0;
  auxT.Tvc  = 0;
  auxT.Tjnb = Tjnb;
  auxT.Tacb = Tacb;
  auxT.Tjpb = Tjpb;

  if (tpVerifyTimes(DISTANCE_TOLERANCE, GD,Jerks,IC,Dir,auxT,&FC,&TAccV,&TVelV,&TPosV,TP_OFF)!=TP_OK) {
    /* Stop Motion */
    if(tpStopPosition(IC, Limits, &SD, &auxT) != TP_OK) {
      fprintf(stderr,"tp Jerk PEStp - TP_ERROR - Stop Position     \n  ");
      return TP_ERROR;
    }
  }
  
  if (tpGetMonotonicTimes(auxT,&TM,&NOE)!=TP_OK) {
    fprintf(stderr,"tp Jerks PEStp - TP_ERROR - Get Monotonic Times \n");
    return TP_ERROR;
  }

  if (tpBuildVectorsSD(Dir, Limits, Jerks, IC, TM, TAccV, TVelV, TPosV, Acc, Vel, Pos,kBase)!= TP_OK) {
    fprintf(stderr,"tp Jerks PEStp - TP_ERROR - Build Vectors \n");
    return TP_ERROR;
  }
  
  freemat(VR_Sol);
  return TP_OK;
}
